%% Hierarchisches Update Zufallsfeld eindimensional

clear all; close all; clc;

%{
--------------------------------------------------------------------------
Created by:                         Date:           Comment:
Sebastian Geyer                     Juli 2021         ---
Technische Universitaet Muenchen
s.geyer@tum.de
--------------------------------------------------------------------------
%}

%% Wahl der Modellparameter

% Geometrie
L    = 50;   % Laenge des zu untersuchenden Bereichs [m]
L_el = 0.02; % Laenge der Diskretisierungselemente [m]

% Verteilungstyp des zu untersuchenden Zufallsfelds
RF_dist = 'logN'; % ('N': Normalverteilung, 'logN': Lognormalverteilung)

% Art des Updates (marginal oder multivariat)
BU_type = 'mv'; % 'marg' oder 'mv'

% Darstellung der Ergebnisse
plot_res  = true; % (plot_res = true: Ergebnisse darstellen; plot_res = false: Ergebnisse nicht darstellen)
plot_real = 3; % (Anzahl der Realisierungen, die illustriert werden sollen)

% Korrelationsmodell
nu    = 2.5; % Smoothness-Parameter des Matern-Modells (nur fuer Werte von nu = N+0.5 implementiert (N positiv und ganzzahlig), zus.: nu = inf (square-exp. Modell))
L_c   = NaN; % Korrelationslaenge [m] (L_c = NaN: Erlernen der Korrelationslaenge)
g_mic = NaN; % Anteil der Mikrokorrelation (0 <= g_mic <= 1; g_mic = NaN: Erlernen des Anteils)

% A-priori-Parameter der Normal-Gamma-Verteilung (Noninformative a-priori-Parameter: mu0 = 0, k0 = 0, a0 = -1/2, b0 = 0)
mu0 = 0;    % Mittelwert der Normalverteilung
k0  = 0;    % Skalierungsfaktor fr precision der Normalverteilung
a0  = -1/2; % Formparameter der Gammaverteilung
b0  = 0;    % Inverser Skalenparameter der Gammaverteilung

% Parameter der Messungenauigkeit
switch RF_dist
    case 'N'    % (normal, additiv)
        si_eps = 0; % (si_eps = 0: Keine Messungenauigkeit)
    case 'logN' % (lognormal, multiplikativ)
        CV_eps     = 0; % Variationskoeffizient (CV_eps = 0: Keine Messungenauigkeit)
        si_eps_log = sqrt(log(1+CV_eps^2)); % Standardabweichung der zugrundeliegenden Normalverteilung (si_eps*si_X)
end

%% Datenimport

% Einlesen der Daten aus Excel-Datei
% ID | fc [MPa] | z [m]
data = readmatrix('Daten_dummy_test_1D.xlsx','Sheet','Daten_DF','Range','A2:C11');

% Log-Transformation, falls ntig
switch RF_dist
    case 'N'
        data_update = data(:,2:3);
    case 'logN'
        data_update =  [log(data(:,2)) data(:,3)];
end

%% Ueberpruefung der Eingabe

if ~strcmp(RF_dist,'N') && ~strcmp(RF_dist,'logN')
    error('Verteilungstyp nicht korrekt definiert, bitte ueberpruefen')
elseif ~strcmp(BU_type,'marg') && ~strcmp(BU_type,'mv')
    error('Art des Updates nicht korrekt definiert, bitte ueberpruefen')
elseif ~islogical(plot_res)
    error('Art der Ergebnisdarstellung nicht korrekt definiert, bitte ueberpruefen')
elseif mod(nu-0.5,1)~=0 && nu~=inf
    error('Smoothness-Parameter nicht korrekt definiert, bitte ueberpruefen')
elseif L_c <0
    error('Korrelationslaenge nicht korrekt definiert, bitte ueberpruefen')
elseif g_mic <0 || g_mic >1
    error('Anteil der Mikrokorrelation nicht korrekt definiert, bitte ueberpruefen')
else
    fprintf('\nEingabeparameter in Ordnung\n\n')
end

%%  Erlernen des Korrelationsmodells

R_fun_meso = Matern_corr(nu); % Korrelationsfunktion

if isnan(L_c) || isnan(g_mic)
    
    fprintf('\nMAP-Verfahren zur Abschaetzung der Korrelationsfunktion laeuft...\n')
    
    % Abspeichern der Parameter
    MAP_par.mu0 = mu0;
    MAP_par.k0  = k0;
    MAP_par.a0  = a0;
    MAP_par.b0  = b0;
    MAP_par.R_fun_meso = R_fun_meso;
    MAP_par.M  = data_update;
    switch RF_dist
        case 'N'
            MAP_par.si_eps = si_eps;
        case 'logN'
            MAP_par.si_eps = si_eps_log;
    end
    
    % Lsung des Optimisierungsproblems (maximum-a-posteriori)
    n_opt   = 10; % Anzahl der Minimisierungsversuche
    val_MAP = zeros(n_opt,1);
    if plot_res
        n_plot  = 50; % Auflsung der Ergebnisdarstellung
    end
    
    if isnan(L_c) && ~isnan(g_mic) % Erlernen der Korrelationslnge
        
        sol_MAP = zeros(n_opt,1);
        
        MAP_par.learn='lc';
        MAP_par.g_mic=g_mic;
        save('MAP_par','MAP_par');
        
        L_c_bounds = [0 20]; % Grenzen der Optimisierungsinitialisierung
        for ii = 1:n_opt
            [sol_MAP(ii),val_MAP(ii)] = fminsearch(@f_MAP_1D,(ii-1)*range(L_c_bounds)/(n_opt-1)+min(L_c_bounds));
        end
        [~,id_MAP] = min(val_MAP);
        L_c         = round(sol_MAP(id_MAP,1),2);
        
        fprintf('\nL_c ueber MAP-Schaetzer: L_c = %.2f m\n',L_c)
        
        % Darstellung des Optimisierungsproblems
        if plot_res
            L_c_eval = linspace(0,20,n_plot);
            MAP_eval = zeros (n_plot,1);
            for ii =1:length(L_c_eval)
                MAP_eval(ii) = f_MAP_1D(L_c_eval(ii));
            end
            figure; hold on
            plot(L_c_eval,exp(-0.5*MAP_eval),'r','LineWidth',1)
            line([L_c L_c],[0 exp(-0.5*val_MAP(id_MAP))],'Color','k','LineStyle','--');
            plot(L_c,0,'k.','MarkerSize',15)
            plot(L_c,exp(-0.5*val_MAP(id_MAP)),'k.','MarkerSize',15)
            box on
            grid on
            xlabel('$L_c$','Interpreter','latex')
            ylabel('$f(L_c|\mathbf{M})$','Interpreter','latex')
            title('A-posteriori-Verteilung von $L_c$ mit MAP-Schaetzer','Interpreter','latex')
        end
        
    elseif ~isnan(L_c) && isnan(g_mic) % Erlernen der Mikrokorrelation
        
        sol_MAP = zeros(n_opt,1);
        
        MAP_par.learn='g_mic';
        MAP_par.lc=L_c;
        save('MAP_par','MAP_par');
        
        for ii = 1:n_opt
            [sol_MAP(ii),val_MAP(ii)] = fminsearch(@f_MAP_1D,(ii-1)/(n_opt-1));
        end
        [~,id_MAP] = min(val_MAP);
        g_mic         = round(sol_MAP(id_MAP,1),2);
        
        fprintf('\ng_mic ueber MAP-Schaetzer: g_mic = %.2f\n',g_mic)
        
        % Darstellung des Optimisierungsproblems
        if plot_res
            g_mic_eval = linspace(0,1,n_plot);
            MAP_eval = zeros (n_plot,1);
            for ii =1:length(g_mic_eval)
                MAP_eval(ii) = f_MAP_1D(g_mic_eval(ii));
            end
            figure; hold on
            plot(g_mic_eval,exp(-0.5*MAP_eval),'r','LineWidth',1)
            line([g_mic g_mic],[0 exp(-0.5*val_MAP(id_MAP))],'Color','k','LineStyle','--');
            plot(g_mic,0,'k.','MarkerSize',15)
            plot(g_mic,exp(-0.5*val_MAP(id_MAP)),'k.','MarkerSize',15)
            box on
            grid on
            xlabel('$\gamma_{\textrm{micro}}$','Interpreter','latex')
            ylabel('$f(\gamma_{\textrm{micro}}|\mathbf{M})$','Interpreter','latex')
            title('A-posteriori-Verteilung von $\gamma_{\textrm{micro}}$ mit MAP-Schaetzer','Interpreter','latex')
        end
        
    else % Erlernen der Korrelationslnge und der Mikrokorrelation
        
        save('MAP_par','MAP_par');
        
        L_c_bounds = [0 20]; % Grenzen der Optimisierungsinitialisierung
        for ii = 1:n_opt
            [sol_MAP(ii,:),val_MAP(ii)] = fminsearch(@f_MAP_1D,[rand*range(L_c_bounds)+min(L_c_bounds),rand]);
        end
        [~,id_MAP] = min(val_MAP);
        L_c        = round(sol_MAP(id_MAP,1),2);
        g_mic      = round(sol_MAP(id_MAP,2),2);
        
        fprintf('\nL_c ueber MAP-Schaetzer: L_c = %.2f m\n',L_c)
        fprintf('g_mic ueber MAP-Schaetzer: g_mic = %.2f\n',g_mic)
        
        % Darstellung des Optimisierungsproblems
        if plot_res
            n_plot=50;
            L_c_eval = linspace(eps,20,n_plot);
            g_mic_eval = linspace(0,1,n_plot);
            MAP_eval = zeros(n_plot);
            
            [z1,z2]=meshgrid(L_c_eval,g_mic_eval');
            
            for ii = 1:n_plot
                for jj= 1:n_plot
                    MAP_eval(ii,jj)=f_MAP_1D([z1(ii,jj),z2(ii,jj)]);
                end
            end
            
            figure; hold on
            surf(z1,z2,exp(-0.5*MAP_eval))
            ax=gca;
            axis tight;
            colormap(hot)
            plot3(L_c,g_mic,exp(-0.5*val_MAP(id_MAP)),'k.','MarkerSize',25)
            plot3(L_c,0,0,'k.','MarkerSize',15)
            plot3(0,g_mic,0,'k.','MarkerSize',15)
            line([L_c L_c],[0 g_mic],[0 0],'Color','black','LineStyle','--')
            line([0 L_c],[g_mic g_mic],[0 0],'Color','black','LineStyle','--')
            line([L_c L_c],[g_mic g_mic],[0 exp(-0.5*val_MAP(id_MAP))],'Color','black','LineStyle','--')
            box on
            set(gcf, 'color', [1 1 1])
            grid on
            ax.XMinorGrid='on';
            ax.YMinorGrid='on';
            ax.ZMinorGrid='on';
            xlabel('$L_c$','Interpreter','latex')
            y_lab=ylabel('$\gamma_{\textrm{micro}}$','Interpreter','latex');
            zlabel('$f(L_c,\gamma_{\textrm{micro}}|\mathbf{M})$','Interpreter','latex')
            view(-43,15)
            title('A-posteriori-Verteilung von $L_c$ und $\gamma_{\textrm{micro}}$ mit MAP-Schaetzer','Interpreter','latex')
            
        end
        
    end
    
end

R_fun = @(Dw)g_mic*(Dw==0)+(1-g_mic)*R_fun_meso(Dw);

%% Bayes'sches Update des Zufallsfelds

fprintf('\n\nBayes''sches Update des Zufallsfelds laueft...\n')

% Diskretisierung
z = (L_el/2:L_el:L)';
obj_RF.L = L; % Abspeichern der Lnge des Zufallsfelds

% Parameter der posterior predictive Verteilung
if strcmp(RF_dist,'N')
    obj_RF.RF_dist = 't'; % Student's t-Verteilung als posterior predictive Verteilung
    [mu_t,lambda_t_inv,nu_t,NG_pp] = t_postpred(mu0,k0,a0,b0,R_fun,data_update,z,si_eps,L_c,BU_type);
else
    obj_RF.RF_dist = 'logt'; % log-Student's t-Verteilung als posterior predictive Verteilung
    [mu_t,lambda_t_inv,nu_t,NG_pp] = t_postpred(mu0,k0,a0,b0,R_fun,data_update,z,si_eps_log,L_c,BU_type);
end

if strcmp(BU_type,'marg')
    fprintf('\nMarginales Update erfolgreich\n')
    si_t=sqrt(lambda_t_inv*nu_t/(nu_t-2)); % Standardabweichung
    obj_RF.type = 'marg'; % Nur marginale Parameter gespeichert
else
    fprintf('\nMultivariates Update erfolgreich\n')
    Sigma_t = lambda_t_inv*nu_t/(nu_t-2); % Kovarianzmatrix
    si_t    = sqrt(diag(Sigma_t)); % Standardabweichung
    obj_RF.type = 'mv'; % multivariate Parameter gespeichert
end

%% Post-processing

% Definition der Funktion zur Generierung von Stichproben
if strcmp(BU_type,'marg') % nur marginale Stichproben ohne Korrelation
    
    if strcmp(obj_RF.RF_dist,'t') % Student's t-Zufallsfeld
        obj_RF.f_rand = @(n_rand)(ones(length(z),1)*(nu_t./chi2rnd(nu_t,1,n_rand)).^(1/2)).*mvnrnd(zeros(1,length(z)),diag(si_t.^2),n_rand)'+mu_t;
    else % log-Student's t-Zufallsfeld
        obj_RF.f_rand = @(n_rand)exp((ones(length(z),1)*(nu_t./chi2rnd(nu_t,1,n_rand)).^(1/2)).*mvnrnd(zeros(1,length(z)),diag(si_t.^2),n_rand)'+mu_t);
    end
    
else % Stichproben des Zufallsfelds unter Bercksichtigung der Korrelation
    
    if strcmp(obj_RF.RF_dist,'t') % Student's t-Zufallsfeld
        obj_RF.f_rand = @(n_rand)(ones(length(z),1)*(nu_t./chi2rnd(nu_t,1,n_rand)).^(1/2)).*mvnrnd(zeros(1,length(z)),Sigma_t,n_rand)'+mu_t;
    else % log-Student's t-Zufallsfeld
        obj_RF.f_rand = @(n_rand)exp((ones(length(z),1)*(nu_t./chi2rnd(nu_t,1,n_rand)).^(1/2)).*mvnrnd(zeros(1,length(z)),Sigma_t,n_rand)'+mu_t);
    end
    
end

% Diskretisierungspunkte
obj_RF.z_eval = z;

% Parameter der Student's t-Verteilung
obj_RF.mu_t = mu_t;
obj_RF.lambda_t_inv = lambda_t_inv;
obj_RF.nu_t = nu_t;

% Abspeichern der Variable
save('obj_RF','obj_RF');

fprintf('\n\nErgebnis in Variable ''obj_RF.mat'' abgespeichert\n')

%% Darstellung der Ergebnisse

if plot_res
    
    % 5%-Quantilwert und 95%-Quantilwert
    u_5p=tinv(0.05,nu_t);
    if strcmp(RF_dist,'N')
        x_5p=mu_t+sqrt((nu_t-2)/(nu_t))*si_t.*u_5p;
        x_95p=mu_t-sqrt((nu_t-2)/(nu_t))*si_t.*u_5p;
    else
        x_5p=exp(mu_t+sqrt((nu_t-2)/(nu_t))*si_t.*u_5p);
        x_95p=exp(mu_t-sqrt((nu_t-2)/(nu_t))*si_t.*u_5p);
    end
    
    % Median
    if strcmp(RF_dist,'N')
        x_m=mu_t;
    else
        x_m=exp(mu_t);
    end
    
    % Einstellung der Figure
    figure; hold on
    set(gcf,'renderer','Painters')
    set(gcf,'PaperOrientation','landscape');
    
    % Plot der Ergebnisse
    plot(z,x_m,'Color','b','LineWidth',1)
    plot(data(:,3),data(:,2),'o','Color','r','MarkerFaceColor','r');
    plot(z,x_5p,'Color','k','LineWidth',1)
    plot(z,x_95p,'Color','k','LineWidth',1)
    filled_area=fill([z;flipud(z)],[x_5p;flipud(x_95p)],[0.5 0.5 0.5]);
    set(filled_area,'facealpha',.35);
    legend('Median','Datenpunkte','90%-Intervallgrenzen','Location','SouthOutside','Orientation','horizontal')
    set(gca,'children',flipud(get(gca,'children')))
    box on;
    grid on;
    xlabel('Position [m]')
    ylabel('Druckfestigkeit [MPa]')
    title('\textit{Posterior predictive} Verteilung des Zufallsfelds','Interpreter','latex')
    
end

if plot_real>0
    
    % Einstellung der Figure
    figure; hold on
    set(gcf,'renderer','Painters')
    set(gcf,'PaperOrientation','landscape');
    
    x_real = obj_RF.f_rand(plot_real);
    
    cmap=lines(7);
    
    % Plot der Realisierungen
    for ii = 1:plot_real
        plot(z,x_real(:,ii),'Color',cmap(mod(ii,7),:),'LineWidth',1)
        
    end
    box on;
    grid on;
    xlabel('Position [m]')
    ylabel('Druckfestigkeit [MPa]')
    if strcmp(obj_RF.type,'marg')
        title('Stichproben der marginalen \textit{posterior predictive} Verteilung des Zufallsfelds [MPa]','Interpreter','latex')
    else
        title('Stichproben des \textit{posterior predictive} Zufallsfelds [MPa]','Interpreter','latex')
    end
end

%%% ENDE