%% Hierarchisches Update Zufallsfeld mehrdimensional

clear all; close all; clc;

%{
--------------------------------------------------------------------------
Created by:                         Date:           Comment:
Sebastian Geyer                     Juli 2021         ---
Technische Universitaet Muenchen
s.geyer@tum.de
--------------------------------------------------------------------------
%}

%% Wahl der Modellparameter

% Geometrie
L    = [132 8];  % Laenge des zu untersuchenden Bereichs je Dimension [m]
L_el = [0.25 0.1];           % Laenge der Diskretisierungselemente [m] (Skalar bei gleicher Laenge je Dimension, Zeilenvektor 1 x dim bei unterschiedlichen Laengen)

% Verteilungstyp des zu untersuchenden Zufallsfelds
RF_dist = 'logN'; % ('N': Normalverteilung, 'logN': Lognormalverteilung)

% Art des Updates (marginal oder multivariat)
BU_type = 'marg'; % 'marg' oder 'mv'

% Darstellung der Ergebnisse
plot_res  = true; % (plot_res = true: Ergebnisse darstellen; plot_res = false: Ergebnisse nicht darstellen)
plot_real = 0;    % (Anzahl der Realisierungen, die illustriert werden sollen)

% Korrelationsmodell
nu    = 0.5;         % Smoothness-Parameter des Matern-Modells (nur fuer Werte von nu = N+0.5 implementiert (N positiv und ganzzahlig), zus.: nu = inf (square-exp. Modell))
L_c   = [NaN NaN]; % Korrelationslaenge [m] (Zeilenvektor dim x 1; L_c(i) = NaN: Erlernen der Korrelationslaenge in Richtung i; Kombinationen moeglich)
g_mic = NaN;         % Anteil der Mikrokorrelation (0 <= g_mic <= 1; g_mic = NaN: Erlernen des Anteils)
iso   = [1 2];     % Isotropie der Korrelationsfunktion (gleicher Index fuer isotrope Richtungen)

% A-priori-Parameter der Normal-Gamma-Verteilung (Noninformative a-priori-Parameter: mu0 = 0, k0 = 0, a0 = -1/2, b0 = 0)
mu0 = 0;    % Mittelwert der Normalverteilung
k0  = 0;    % Skalierungsfaktor fr precision der Normalverteilung
a0  = -1/2; % Formparameter der Gammaverteilung
b0  = 0;    % Inverser Skalenparameter der Gammaverteilung

% Parameter der Messungenauigkeit
switch RF_dist
    case 'N'    % (normal, additiv)
        si_eps = eps; % (si_eps = 0: Keine Messungenauigkeit)
    case 'logN' % (lognormal, multiplikativ)
        CV_eps = eps; % Variationskoeffizient (CV_eps = 0: Keine Messungenauigkeit)
        si_eps_log = sqrt(log(1+CV_eps^2)); % Standardabweichung der zugrundeliegenden Normalverteilung (si_eps*si_X)
end

%% Datenimport

% Einlesen der Daten aus Excel-Datei
% ID | fc [MPa] | z_1 [m] | z_2 [m] | z_3 [m] (in 3D)
data = readmatrix('Daten_Schleuse_Oldenburg.xlsx','Sheet','Daten_DF2D','Range','A2:D25');

% Log-Transformation, falls ntig
switch RF_dist
    case 'N'
        data_update = data(:,2:end);
    case 'logN'
        data_update =  [log(data(:,2)) data(:,3:end)];
end

%% Ueberpruefung der Eingabe

if ~strcmp(RF_dist,'N') && ~strcmp(RF_dist,'logN')
    error('Verteilungstyp nicht korrekt definiert, bitte ueberpruefen')
elseif ~strcmp(BU_type,'marg') && ~strcmp(BU_type,'mv')
    error('Art des Updates nicht korrekt definiert, bitte ueberpruefen')
elseif ~islogical(plot_res)
    error('Art der Ergebnisdarstellung nicht korrekt definiert, bitte ueberpruefen')
elseif mod(nu-0.5,1)~=0 && nu~=inf
    error('Smoothness-Parameter nicht korrekt definiert, bitte ueberpruefen')
elseif isnumeric(L_c) && min(L_c) <0
    error('Korrelationslaenge nicht korrekt definiert, bitte ueberpruefen')
elseif g_mic <0 || g_mic >1
    error('Anteil der Mikrokorrelation nicht korrekt definiert, bitte ueberpruefen')
elseif (size(data,2)-2)~=size(L,2)
    error('Dimensionen der Daten stimmt nicht mit den Dimensionen der Zufallsfelddefinition ueberein, bitte ueberpruefen')
elseif length(L_c)~=length(L)
    error('Dimensionen der Korrelationslaenge stimmt nicht mit den Dimensionen der Zufallsfelddefinition ueberein, bitte ueberpruefen')
elseif length(iso)~=length(L_c)
    error('Dimensionen der Isotropie stimmt nicht mit den Dimensionen der Zufallsfelddefinition ueberein, bitte ueberpruefen')
elseif numel(unique(iso))<length(L_c)
    for ii = 1:length(L_c)
        id_iso = find(iso==iso(ii));
        if length(id_iso)>1
            L_c_iso = L_c(id_iso);
            if ~all(isnan(L_c_iso))
                if any(isnan(L_c_iso))
                    error('Definition der Isotropie nicht kompatibel mit Definition der Korrelationslaengen, bitte ueberpruefen')
                elseif length(unique(L_c_iso))>1
                    error('Definition der Isotropie nicht kompatibel mit Definition der Korrelationslaengen, bitte ueberpruefen')
                end
            end
            
        end
    end
end

fprintf('\nEingabeparameter in Ordnung\n')

%%  Erlernen des Korrelationsmodells

R_fun_meso = Matern_corr(nu); % Korrelationsfunktion

if any(isnan(L_c)) || isnan(g_mic)
    
    opts = optimset('Display','off');
    
    fprintf('\n\nMAP-Verfahren zur Abschaetzung der Korrelationsfunktion laeuft...\n')
    
    % Abspeichern der Parameter
    MAP_par.mu0 = mu0;
    MAP_par.k0  = k0;
    MAP_par.a0  = a0;
    MAP_par.b0  = b0;
    MAP_par.R_fun_meso = R_fun_meso;
    MAP_par.M  = data_update;
    switch RF_dist
        case 'N'
            MAP_par.si_eps = si_eps;
        case 'logN'
            MAP_par.si_eps = si_eps_log;
    end
    
    % Lsung des Optimisierungsproblems (maximum-a-posteriori)
    n_opt   = 10; % Anzahl der Minimisierungsversuche
    val_MAP = zeros(n_opt,1);
    if plot_res
        n_plot  = 50; % Auflsung der Ergebnisdarstellung
    end
    
    if ~isnan(g_mic) % Erlernen der Korrelationslnge
        
        id_learn  = find(isnan(L_c));
        iso_learn = iso(id_learn);
        [L_c_learn,idc,idd] = unique(iso_learn,'stable');
        n_learn = length(idc);
        [L_c_diff,ida,idb] = unique(iso,'stable');
        
        sol_MAP = zeros(n_opt,n_learn);
        
        MAP_par.learn='L_c';
        MAP_par.L_c = L_c;
        MAP_par.id_learn=id_learn;
        MAP_par.iso_learn = iso_learn;
        MAP_par.g_mic=g_mic;
        save('MAP_par','MAP_par');
        
        % Grenzen der Optimisierungsinitialisierung
        L_c_bounds = zeros(2,n_learn);
        for ii=1:n_learn
            id_L = iso==L_c_diff(ii);
            L_c_bounds(2,ii) = max(L(id_L));
        end
        
        % Optimisierungsalgorithmus
        for ii = 1:n_opt
            [sol_MAP(ii,:),val_MAP(ii)] = fminsearch(@f_MAP_multiD,(ii-1)*range(L_c_bounds)/(n_opt-1)+min(L_c_bounds),opts);
        end
        [~,id_MAP]      = min(val_MAP);
        L_c_learned     = round(sol_MAP(id_MAP,:),2);
        L_c(MAP_par.id_learn) = L_c_learned(idd);
        
        for ii = 1:length(id_learn)
            fprintf('\nL_c(%.0f) ueber MAP-Schaetzer: L_c = %.2f m',id_learn(ii),L_c(id_learn(ii)))
        end
        
        % Darstellung des Optimisierungsproblems
        if plot_res
            if length(L_c_learned) == 1
                
                L_c_eval = linspace(eps,max(max(L_c_bounds),1.5*L_c_learned),n_plot);
                MAP_eval = zeros (n_plot,1);
                for ii =1:length(L_c_eval)
                    MAP_eval(ii) = f_MAP_multiD(L_c_eval(ii));
                end
                figure; hold on
                plot(L_c_eval,exp(-0.5*MAP_eval),'r','LineWidth',1)
                line([L_c_learned L_c_learned],[0 exp(-0.5*val_MAP(id_MAP))],'Color','k','LineStyle','--');
                plot(L_c_learned,0,'k.','MarkerSize',15)
                plot(L_c_learned,exp(-0.5*val_MAP(id_MAP)),'k.','MarkerSize',15)
                box on
                grid on
                xlabel('$L_c$','Interpreter','latex')
                ylabel('$f(L_c|\mathbf{M})$','Interpreter','latex')
                title('A-posteriori-Verteilung von $L_c$ mit MAP-Schaetzer','Interpreter','latex')
                
            elseif length(L_c_learned)==2
                
                L_c_eval1 = linspace(eps,max(L_c_bounds(2,1),1.5*L_c_learned(1)),n_plot);
                L_c_eval2 = linspace(eps,max(L_c_bounds(2,2),1.5*L_c_learned(2)),n_plot);
                MAP_eval = zeros(n_plot);
                
                [z1,z2]=meshgrid(L_c_eval1,L_c_eval2');
                
                for ii = 1:n_plot
                    for jj= 1:n_plot
                        MAP_eval(ii,jj)=f_MAP_multiD([z1(ii,jj),z2(ii,jj)]);
                    end
                end
                
                figure; hold on
                surf(z1,z2,exp(-0.5*MAP_eval))
                ax=gca;
                axis tight;
                colormap(hot)
                plot3(L_c_learned(1),L_c_learned(2),exp(-0.5*val_MAP(id_MAP)),'k.','MarkerSize',25)
                plot3(L_c_learned(1),0,0,'k.','MarkerSize',15)
                plot3(0,L_c_learned(2),0,'k.','MarkerSize',15)
                line([L_c_learned(1) L_c_learned(1)],[0 L_c_learned(2)],[0 0],'Color','black','LineStyle','--')
                line([0 L_c_learned(1)],[L_c_learned(2) L_c_learned(2)],[0 0],'Color','black','LineStyle','--')
                line([L_c_learned(1) L_c_learned(1)],[L_c_learned(2) L_c_learned(2)],[0 exp(-0.5*val_MAP(id_MAP))],'Color','black','LineStyle','--')
                box on
                set(gcf, 'color', [1 1 1])
                grid on
                ax.XMinorGrid='on';
                ax.YMinorGrid='on';
                ax.ZMinorGrid='on';
                x_text = ['$L_c(',regexprep(num2str(id_learn(idb==1)),'\s+',','),')$'];
                xlabel(x_text,'Interpreter','latex')
                y_text = ['$L_c(',regexprep(num2str(id_learn(idb==2)),'\s+',','),')$'];
                y_lab=ylabel(y_text,'Interpreter','latex');
                zlabel('$f(\boldmath{L_c}|\mathbf{M})$','Interpreter','latex')
                view(-43,15)
                title('A-posteriori-Verteilung von $\boldmath{L_c}$ (nur gelernte $\boldmath{L_c}$) mit MAP-Schaetzer','Interpreter','latex')
                
            else
                fprintf('\nGrafische Ausgabe der gemeisamen a-posteriori-Verteilung von L_c mit MAP-Schaetzer nicht moeglich in 3D\n')
                
                for ii= 1:n_learn
                    
                    L_c_eval = linspace(eps,max(max(L_c_bounds(2,ii)),1.5*L_c_learned(ii)),n_plot);
                    MAP_eval = zeros (n_plot,1);
                    for jj =1:length(L_c_eval)
                        L_c_evaljj = L_c_learned;
                        L_c_evaljj(ii) = L_c_eval(jj);
                        MAP_eval(jj) = f_MAP_multiD(L_c_evaljj);
                    end
                    figure; hold on
                    plot(L_c_eval,exp(-0.5*MAP_eval),'r','LineWidth',1)
                    line([L_c_learned(ii) L_c_learned(ii)],[0 exp(-0.5*val_MAP(id_MAP))],'Color','k','LineStyle','--');
                    plot(L_c_learned(ii),0,'k.','MarkerSize',15)
                    plot(L_c_learned(ii),exp(-0.5*val_MAP(id_MAP)),'k.','MarkerSize',15)
                    box on
                    grid on
                    x_text = ['$L_c(',regexprep(num2str(id_learn(idb==ii)),'\s+',','),')$'];
                    xlabel(x_text,'Interpreter','latex')
                    y_text = ['$f(L_c(',regexprep(num2str(id_learn(idb==ii)),'\s+',','),')|\mathbf{M})$'];
                    ylabel(y_text,'Interpreter','latex')
                    title_text = ['A-posteriori-Verteilung von $L_c(',regexprep(num2str(id_learn(idb==ii)),'\s+',','),')$ mit MAP-Schaetzer'];
                    title(title_text,'Interpreter','latex')
                end
                
            end
            
        end
        
    elseif all(~isnan(L_c)) % Erlernen der Mikrokorrelation
        
        sol_MAP = zeros(n_opt,1);
        
        MAP_par.learn='g_mic';
        MAP_par.L_c=L_c;
        save('MAP_par','MAP_par');
        
        for ii = 1:n_opt
            [sol_MAP(ii),val_MAP(ii)] = fminsearch(@f_MAP_multiD,(ii-1)/(n_opt-1),opts);
        end
        [~,id_MAP] = min(val_MAP);
        g_mic         = round(sol_MAP(id_MAP,1),2);
        
        fprintf('\ng_mic ueber MAP-Schaetzer: g_mic = %.2f\n',g_mic)
        
        % Darstellung des Optimisierungsproblems
        if plot_res
            g_mic_eval = linspace(0,1,n_plot);
            MAP_eval = zeros (n_plot,1);
            for ii =1:length(g_mic_eval)
                MAP_eval(ii) = f_MAP_multiD(g_mic_eval(ii));
            end
            figure; hold on
            plot(g_mic_eval,exp(-0.5*MAP_eval),'r','LineWidth',1)
            line([g_mic g_mic],[0 exp(-0.5*val_MAP(id_MAP))],'Color','k','LineStyle','--');
            plot(g_mic,0,'k.','MarkerSize',15)
            plot(g_mic,exp(-0.5*val_MAP(id_MAP)),'k.','MarkerSize',15)
            box on
            grid on
            xlabel('$\gamma_{\textrm{micro}}$','Interpreter','latex')
            ylabel('$f(\gamma_{\textrm{micro}}|\mathbf{M})$','Interpreter','latex')
            title('A-posteriori-Verteilung von $\gamma_{\textrm{micro}}$ mit MAP-Schaetzer','Interpreter','latex')
        end
        
    else % Erlernen der Korrelationslnge und der Mikrokorrelation
        
        id_learn  = find(isnan(L_c));
        iso_learn = iso(id_learn);
        [L_c_learn,idc,idd] = unique(iso_learn,'stable');
        n_learn = length(idc);
        [L_c_diff,ida,idb] = unique(iso,'stable');
        
        sol_MAP = zeros(n_opt,n_learn+1);
        
        MAP_par.learn='L_c_and_g_mic';
        MAP_par.L_c = L_c;
        MAP_par.id_learn=id_learn;
        MAP_par.iso_learn = iso_learn;
        save('MAP_par','MAP_par');
        
        % Grenzen der Optimisierungsinitialisierung
        L_c_bounds = zeros(2,n_learn);
        for ii=1:n_learn
            id_L = iso==L_c_diff(ii);
            L_c_bounds(2,ii) = max(L(id_L));
        end
        
        % Optimisierungsalgorithmus
        for ii = 1:n_opt
            [sol_MAP(ii,:),val_MAP(ii)] = fminsearch(@f_MAP_multiD,[(ii-1)*range(L_c_bounds)/(n_opt-1)+min(L_c_bounds),rand],opts);
        end
        [~,id_MAP]      = min(val_MAP);
        L_c_learned     = max(round(sol_MAP(id_MAP,1:end-1),2),1e-4);
        L_c(MAP_par.id_learn) = L_c_learned(idd);
        g_mic         = round(sol_MAP(id_MAP,end),2);
        
        for ii = 1:length(id_learn)
            fprintf('\nL_c(%.0f) ueber MAP-Schaetzer: L_c = %.2f m',id_learn(ii),L_c(id_learn(ii)))
        end
        fprintf('\ng_mic ueber MAP-Schaetzer: g_mic = %.2f\n',g_mic)
        
        % Darstellung des Optimisierungsproblems
        if plot_res
            if length(L_c_learned) == 1
                L_c_eval = linspace(eps,max([max(L_c_bounds),1.5*L_c_learned]),n_plot);
                g_mic_eval = linspace(0,1,n_plot);
                MAP_eval = zeros(n_plot);
                
                [z1,z2]=meshgrid(L_c_eval,g_mic_eval');
                
                for ii = 1:n_plot
                    for jj= 1:n_plot
                        MAP_eval(ii,jj)=f_MAP_multiD([z1(ii,jj),z2(ii,jj)]);
                    end
                end
                
                figure; hold on
                surf(z1,z2,exp(-0.5*MAP_eval))
                ax=gca;
                axis tight;
                colormap(hot)
                plot3(L_c_learned,g_mic,exp(-0.5*val_MAP(id_MAP)),'k.','MarkerSize',25)
                plot3(L_c_learned,0,0,'k.','MarkerSize',15)
                plot3(0,g_mic,0,'k.','MarkerSize',15)
                line([L_c_learned L_c_learned],[0 g_mic],[0 0],'Color','black','LineStyle','--')
                line([0 L_c_learned],[g_mic g_mic],[0 0],'Color','black','LineStyle','--')
                line([L_c_learned L_c_learned],[g_mic g_mic],[0 exp(-0.5*val_MAP(id_MAP))],'Color','black','LineStyle','--')
                box on
                set(gcf, 'color', [1 1 1])
                grid on
                ax.XMinorGrid='on';
                ax.YMinorGrid='on';
                ax.ZMinorGrid='on';
                x_text = ['$L_c(',regexprep(num2str(id_learn),'\s+',','),')$'];
                xlabel(x_text,'Interpreter','latex')
                y_lab=ylabel('$\gamma_{\textrm{micro}}$','Interpreter','latex');
                zlabel('$f(L_c,\gamma_{\textrm{micro}}|\mathbf{M})$','Interpreter','latex')
                view(-43,15)
                title('A-posteriori-Verteilung von gelerntem $L_c$ und $\gamma_{\textrm{micro}}$ mit MAP-Schaetzer','Interpreter','latex')
            else
                
                fprintf('\nGrafische Ausgabe der a-posteriori-Verteilung von L_c und g_mic mit MAP-Schaetzer ist nicht moeglich fr mehrdimensionales zu lernendes L_c.\n')
                
                for ii= 1:n_learn
                    
                    L_c_eval = linspace(eps,max([max(L_c_bounds(2,ii)),1.5*L_c_learned(ii)]),n_plot);
                    MAP_eval = zeros (n_plot,1);
                    for jj =1:length(L_c_eval)
                        L_c_evaljj = L_c_learned;
                        L_c_evaljj(ii) = L_c_eval(jj);
                        MAP_eval(jj) = f_MAP_multiD([L_c_evaljj g_mic]);
                    end
                    figure; hold on
                    plot(L_c_eval,exp(-0.5*MAP_eval),'r','LineWidth',1)
                    line([L_c_learned(ii) L_c_learned(ii)],[0 exp(-0.5*val_MAP(id_MAP))],'Color','k','LineStyle','--');
                    plot(L_c_learned(ii),0,'k.','MarkerSize',15)
                    plot(L_c_learned(ii),exp(-0.5*val_MAP(id_MAP)),'k.','MarkerSize',15)
                    box on
                    grid on
                    x_text = ['$L_c(',regexprep(num2str(id_learn(idb==ii)),'\s+',','),')$'];
                    xlabel(x_text,'Interpreter','latex')
                    y_text = ['$f(L_c(',regexprep(num2str(id_learn(idb==ii)),'\s+',','),')|\mathbf{M})$'];
                    ylabel(y_text,'Interpreter','latex')
                    title_text = ['A-posteriori-Verteilung von $L_c(',regexprep(num2str(id_learn(idb==ii)),'\s+',','),')$ mit MAP-Schaetzer'];
                    title(title_text,'Interpreter','latex')
                end
                
                g_mic_eval = linspace(0,1,n_plot);
                MAP_eval = zeros (n_plot,1);
                for ii =1:length(g_mic_eval)
                    MAP_eval(ii) = f_MAP_multiD([L_c_learned g_mic_eval(ii)]);
                end
                figure; hold on
                plot(g_mic_eval,exp(-0.5*MAP_eval),'r','LineWidth',1)
                line([g_mic g_mic],[0 exp(-0.5*val_MAP(id_MAP))],'Color','k','LineStyle','--');
                plot(g_mic,0,'k.','MarkerSize',15)
                plot(g_mic,exp(-0.5*val_MAP(id_MAP)),'k.','MarkerSize',15)
                box on
                grid on
                xlabel('$\gamma_{\textrm{micro}}$','Interpreter','latex')
                ylabel('$f(\gamma_{\textrm{micro}}|\mathbf{M})$','Interpreter','latex')
                title('A-posteriori-Verteilung von $\gamma_{\textrm{micro}}$ mit MAP-Schaetzer','Interpreter','latex')
                
            end
        end
        
    end
    
end

R_fun = @(Dw)g_mic*(Dw==0)+(1-g_mic)*R_fun_meso(Dw);

%% Bayes'sches Update des Zufallsfelds

fprintf('\n\nBayes''sches Update des Zufallsfelds laueft...\n')

if length(L_el)==1
    L_el = L_el*ones(size(L));
end

% Diskretisierung
z1 = (L_el(1)/2:L_el(1):L(1))';
z2 = (L_el(2)/2:L_el(2):L(2))';
if length(L)==3 % zz1 laeuft in Zeilen, zz2 laeuft in Spalten, zz3 laeuft in Schichten
    z3 = (L_el(3)/2:L_el(3):L(3))';
    [zz1,zz2,zz3] = meshgrid(z1,z2,z3);
    z_eval = [zz1(:) zz2(:) zz3(:)];
else % zz1 laeuft in Zeilen, zz2 laeuft in Spalten
    [zz1,zz2] = meshgrid(z1,z2);
    z_eval = [zz1(:) zz2(:)];
end
obj_RF.L = L; % Abspeichern der Lnge des Zufallsfelds

% Parameter der posterior predictive Verteilung
if strcmp(RF_dist,'N')
    obj_RF.RF_dist = 't'; % Student's t-Verteilung als posterior predictive Verteilung
    [mu_t,lambda_t_inv,nu_t,NG_pp] = t_postpred(mu0,k0,a0,b0,R_fun,data_update,z_eval,si_eps,L_c,BU_type);
else
    obj_RF.RF_dist = 'logt'; % log-Student's t-Verteilung als posterior predictive Verteilung
    [mu_t,lambda_t_inv,nu_t,NG_pp] = t_postpred(mu0,k0,a0,b0,R_fun,data_update,z_eval,si_eps_log,L_c,BU_type);
end

if strcmp(BU_type,'marg')
    fprintf('\nMarginales Update erfolgreich\n')
    si_t=sqrt(lambda_t_inv*nu_t/(nu_t-2)); % Standardabweichung
    obj_RF.type = 'marg'; % Nur marginale Parameter gespeichert
else
    fprintf('\nMultivariates Update erfolgreich\n')
    Sigma_t = lambda_t_inv*nu_t/(nu_t-2); % Kovarianzmatrix
    si_t    = sqrt(diag(Sigma_t)); % Standardabweichung
    obj_RF.type = 'mv'; % multivariate Parameter gespeichert
end


%% Post-processing

% Definition der Funktion zur Generierung von Stichproben
if strcmp(BU_type,'marg') % nur marginale Stichproben ohne Korrelation
    
    if strcmp(obj_RF.RF_dist,'t') % Student's t-Zufallsfeld
        obj_RF.f_rand = @(n_rand)(ones(size(z_eval,1),1)*(nu_t./chi2rnd(nu_t,1,n_rand)).^(1/2)).*mvnrnd(zeros(1,size(z_eval,1)),diag(si_t.^2),n_rand)'+mu_t;
    else % log-Student's t-Zufallsfeld
        obj_RF.f_rand = @(n_rand)exp((ones(size(z_eval,1),1)*(nu_t./chi2rnd(nu_t,1,n_rand)).^(1/2)).*mvnrnd(zeros(1,size(z_eval,1)),diag(si_t.^2),n_rand)'+mu_t);
    end
    
else % Stichproben des Zufallsfelds unter Bercksichtigung der Korrelation
    
    if strcmp(obj_RF.RF_dist,'t') % Student's t-Zufallsfeld
        obj_RF.f_rand = @(n_rand)(ones(size(z_eval,1),1)*(nu_t./chi2rnd(nu_t,1,n_rand)).^(1/2)).*mvnrnd(zeros(1,size(z_eval,1)),Sigma_t,n_rand)'+mu_t;
    else % log-Student's t-Zufallsfeld
        obj_RF.f_rand = @(n_rand)exp((ones(size(z_eval,1),1)*(nu_t./chi2rnd(nu_t,1,n_rand)).^(1/2)).*mvnrnd(zeros(1,size(z_eval,1)),Sigma_t,n_rand)'+mu_t);
    end
    
end

% Diskretisierungspunkte
obj_RF.z_eval = z_eval;

% Parameter der Student's t-Verteilung
obj_RF.mu_t = mu_t;
obj_RF.lambda_t = lambda_t_inv;
obj_RF.nu_t = nu_t;

% Abspeichern der Variable
save('obj_RF','obj_RF');

fprintf('\n\nErgebnis in Variable ''obj_RF.mat'' abgespeichert\n')

%% Darstellung der Ergebnisse

if plot_res
    if length(L)==2
        % 5%-Quantilwert und 95%-Quantilwert
        u_5p=tinv(0.05,nu_t);
        if strcmp(RF_dist,'N')
            x_5p=mu_t+sqrt((nu_t-2)/(nu_t))*si_t.*u_5p;
            x_95p=mu_t-sqrt((nu_t-2)/(nu_t))*si_t.*u_5p;
        else
            x_5p=exp(mu_t+sqrt((nu_t-2)/(nu_t))*si_t.*u_5p);
            x_95p=exp(mu_t-sqrt((nu_t-2)/(nu_t))*si_t.*u_5p);
        end
        
        % Median
        if strcmp(RF_dist,'N')
            x_m=mu_t;
        else
            x_m=exp(mu_t);
        end
        
        x_m_plot   = reshape(x_m,size(zz1));
        x_5p_plot  = reshape(x_5p,size(zz1));
        x_95p_plot = reshape(x_95p,size(zz1));
        
        min_x = min([x_m x_5p x_95p],[],'all');
        max_x = max([x_m x_5p x_95p],[],'all');
        
        % Einstellung der Figure
        figure; hold on
        set(gcf,'renderer','Painters')
        set(gcf,'PaperOrientation','landscape');
        
        % Plot des Medians
        [~,obj_plot_xm]=contourf(zz1,zz2,x_m_plot);
        obj_plot_xm.LineStyle='-';
        obj_plot_xm.LineWidth=0.1;
        plot(data(:,3),data(:,4),'x','Color','blue','MarkerSize',10,'LineWidth',1.5)
        axis equal tight;
        colormap(flipud(hot))
        colorbar
        caxis([floor(min_x ) ceil(max_x)])
        box on
        set(gcf, 'color', [1 1 1])
        xlabel('$x_1$ [m]','Interpreter','latex')
        ylabel('$x_2$ [m]','Interpreter','latex')
        title('\textit{Posterior predictive} Median des Zufallsfelds [MPa]','Interpreter','latex')
        
        % Einstellung der Figure
        figure; hold on
        set(gcf,'renderer','Painters')
        set(gcf,'PaperOrientation','landscape');
        
        % Plot des 5%-Quantils
        [~,obj_plot_x5p]=contourf(zz1,zz2,x_5p_plot);
        obj_plot_x5p.LineStyle='-';
        obj_plot_x5p.LineWidth=0.1;
        plot(data(:,3),data(:,4),'x','Color','blue','MarkerSize',10,'LineWidth',1.5)
        axis equal tight;
        colormap(flipud(hot))
        colorbar
        caxis([floor(min_x ) ceil(max_x)])
        box on
        set(gcf, 'color', [1 1 1])
        xlabel('$x_1$ [m]','Interpreter','latex')
        ylabel('$x_2$ [m]','Interpreter','latex')
        title('\textit{Posterior predictive} 5\%-Quantilwert des Zufallsfelds [MPa]','Interpreter','latex')
        
        % Einstellung der Figure
        figure; hold on
        set(gcf,'renderer','Painters')
        set(gcf,'PaperOrientation','landscape');
        
        % Plot des 95%-Quantils
        [~,obj_plot_x95p]=contourf(zz1,zz2,x_95p_plot);
        obj_plot_x95p.LineStyle='-';
        obj_plot_x95p.LineWidth=0.1;
        plot(data(:,3),data(:,4),'x','Color','blue','MarkerSize',10,'LineWidth',1.5)
        axis equal tight;
        colormap(flipud(hot))
        colorbar
        caxis([floor(min_x ) ceil(max_x)])
        box on
        set(gcf, 'color', [1 1 1])
        xlabel('$x_1$ [m]','Interpreter','latex')
        ylabel('$x_2$ [m]','Interpreter','latex')
        title('\textit{Posterior predictive} 95\%-Quantilwert des Zufallsfelds [MPa]','Interpreter','latex')
        
    else
        fprintf('\n\nGrafische Ausgabe der Quantilwerte des posterior predictive Zufallsfelds ist nicht moeglich in 3D.\n')
    end
end

if plot_real>0
    if length(L)==2
        
        x_real = obj_RF.f_rand(plot_real);
        
        min_x_real = min(x_real,[],'all');
        max_x_real = max(x_real,[],'all');
        
        % Plot der Realisierungen
        for jj = 1:plot_real
            
            x_real_plot   = reshape(x_real(:,jj),size(zz1));
            
            % Einstellung der Figure
            figure; hold on
            set(gcf,'renderer','Painters')
            set(gcf,'PaperOrientation','landscape');
            [~,obj_plot_x_real]=contourf(zz1,zz2,x_real_plot);
            obj_plot_x_real.LineStyle='-';
            obj_plot_x_real.LineWidth=0.1;
            axis equal tight;
            colormap(flipud(cool))
            colorbar
            caxis([floor(min_x_real) ceil(max_x_real)])
            box on
            set(gcf, 'color', [1 1 1])
            xlabel('$x_1$ [m]','Interpreter','latex')
            ylabel('$x_2$ [m]','Interpreter','latex')
            title('Stichprobe des \textit{posterior predictive} Zufallsfelds [MPa]','Interpreter','latex')
            if strcmp(obj_RF.type,'marg')
                title('Stichprobe der marginalen \textit{posterior predictive} Verteilung des Zufallsfelds','Interpreter','latex')
            else
                title('Stichprobe des \textit{posterior predictive} Zufallsfelds','Interpreter','latex')
            end
        end
        
    else
        fprintf('\nGrafische Ausgabe der Stichproben des posterior predictive Zufallsfelds ist nicht moeglich in 3D.\n')
    end
end

%%% ENDE