%% Druckspannungsnachweis mit raeumlicher Variabilitaet in einer unbewehrten Betonwand

clear all; close all; clc;

%{
--------------------------------------------------------------------------
Created by:                         Date:           Comment:
Sebastian Geyer                     August 2021       ---
Technische Universitaet Muenchen
s.geyer@tum.de
--------------------------------------------------------------------------
%}

%% Wahl der geometrischen Modellparameter

% Abmessungen des Bauwerks
L = 128; % [m] % Gesamtlaenge
H = 9.9; % [m] % Hoehe ab Sohle aufwaerts

% Orte der Nachweisfuehrung
z_1 = 0:1:L; % [m] entlang des Bauwerks
z_3 = 0:0.25:H;  % [m] ueber die Hoehe des Bauwerks (aufwaerts positiv ab Sohle)

%% Wahl der Parameter fuer die Nachweisfuehrung

% Lastfaelle
% G:  Eigengewicht
% W:  Wasserdruck (horizontal und vertikal)
% P:  Porenwasserdruck
% E:  Erddruck
% T:  Verkehrslast
% F:  Wandreibung Erddruck
% A:  Verstaerkungsanker
loadcase = ['G';'W';'P';'E';'T';'F';'A'];

% Auswahl der Teilsicherheitsbeiwerte fuer die Einwirkungen
% 1: Char. Werte der Einwirkung
% 2: DIN 19702 (zwei Faelle, 21 und 22)
% 3: TbW (zwei Faelle, 31 und 32)
vercase = 21;

% Teilsicherheitsbeiwerte und Abminderungsfaktoren fuer den Widerstand
gamma_c = 1.4; % Teilsicherheitsbeiwert Beton (Tab. 8, TbW)
alpha_cc_pl = 0.85; % Uebertragungsfaktor fuer unbewehrten Beton (DIN EN 1992-1-1, 3.1.6 u. 12.3.1)
K_conc = 0.85/1.4; % Koeffizient mit allen erforderlichen Multiplikatoren (f_cd = f_ck*K_conc)

%% Wahl der Zufallsfeldparameter und Datenimport

% Verteilungstyp des zu untersuchenden Zufallsfelds
RF_dist = 'logN'; % ('N': Normalverteilung, 'logN': Lognormalverteilung)

% Korrelationsmodell
nu    = 2.5; % Smoothness-Parameter des Matern-Modells (nur fuer Werte von nu = N+0.5 implementiert (N positiv und ganzzahlig), zus.: nu = inf (square-exp. Modell))
L_c   = [5 5 1]; % Korrelationslaenge [m] (1 x 3 Vektor)
g_mic = 0.25; % Anteil der Mikrokorrelation (0 <= g_mic <= 1)
R_fun_meso = Matern_corr(nu); % Mesoskala-Korrelation
R_fun = @(Dw)g_mic*(Dw==0)+(1-g_mic)*R_fun_meso(Dw); % Korrelationsfunktion

% A-priori-Parameter der Normal-Gamma-Verteilung (Noninformative a-priori-Parameter: mu0 = 0, k0 = 0, a0 = -1/2, b0 = 0)
mu0 = 0;    % Mittelwert der Normalverteilung
k0  = 0;    % Skalierungsfaktor fuer precision der Normalverteilung
a0  = -1/2; % Formparameter der Gammaverteilung
b0  = 0;    % Inverser Skalenparameter der Gammaverteilung

% Parameter der Messungenauigkeit
switch RF_dist
    case 'N'    % (normal, additiv)
        si_eps = 0; % (si_eps = 0: Keine Messungenauigkeit)
    case 'logN' % (lognormal, multiplikativ)
        CV_eps     = 0; % Variationskoeffizient (CV_eps = 0: Keine Messungenauigkeit)
        si_eps_log = sqrt(log(1+CV_eps^2)); % Standardabweichung der zugrundeliegenden Normalverteilung (si_eps*si_X)
end

% Einlesen der Daten aus Excel-Datei (dreidimensionales KOSY erforderlich)
% ID | fc [MPa] | z_1 [m] | z_2 [m] | z_3 [m]
data = readmatrix('Daten_dummy_Nachweis_raeuml.xlsx','Sheet','Daten_DF','Range','A2:E13');

% Anzahl Stuetzstellen fuer SA Integration entlang z_2
n_el = 100;

%% Ueberpruefung der Eingabe

if ~strcmp(RF_dist,'N') && ~strcmp(RF_dist,'logN')
    error('Verteilungstyp nicht korrekt definiert, bitte ueberpruefen')
elseif mod(nu-0.5,1)~=0
    error('Smoothness-Parameter nicht korrekt definiert, bitte ueberpruefen')
elseif any(L_c <0) || any(isnan(L_c))
    error('Korrelationslaenge nicht korrekt definiert, bitte ueberpruefen')
elseif g_mic <0 || g_mic >1 || isnan(g_mic)
    error('Anteil der Mikrokorrelation nicht korrekt definiert, bitte ueberpruefen')
elseif size(data,2)~=5
    error('Messwerte aus externer Datei entsprechen nicht dem erforderlichen Format, bitte ueberpruefen')    
else
    fprintf('\nEingabeparameter des Zufallsfelds in Ordnung\n\n')
end

%% Deterministische Eingansgroessen

fprintf('\nErmittlung der Schnittgroessen und der Querschnittsparameter laeuft...\n')

% Anzahl der Nachweispositionen
n_z_1 = length(z_1); % Anzahl der vertikalen Positionen
n_z_3 = length(z_3); % Anzahl der horizontalen Schnitte

% Ermittlung des Querschnitts, der Schnittgroeen und der Druckzonenhoehe
N      = zeros(n_z_3,1); % [N/m]  Normalkraft (Druck negativ)
V      = zeros(n_z_3,1); % [N/m]  Querkraft (in negativer z_2 Richtung positiv)
M      = zeros(n_z_3,1); % [Nm/m] Biegemoment (gegen den Uhrzeigersinn)
z_2_cs = zeros(n_z_3,2); % [m]    Koordinaten der Endpunkte des Querschnitts in der jeweiligen Hoehe

for ii=1:n_z_3
    [N(ii),V(ii),M(ii),z_2_cs(ii,:)]=SQR_QS_Oldenburg(z_3(ii),loadcase,vercase);
end

h_QS   = z_2_cs(:,2)-z_2_cs(:,1); % [m] Querschnittshoehe
z_2_mp = mean(z_2_cs,2);          % [m] Schwerpunkt
W      = h_QS.^2/6;               % [m^3/m] Widerstandsmoment

% Exzentrizitaet [m]
e   = M./N;
ide = abs(e)<h_QS/6;

% Druckzonenhoehe [m]
h_c       = h_QS.*ones(n_z_3,1);
h_c(~ide) = 3*(h_QS(~ide)/2-abs(e(~ide)));

% sigma_Dreieck
s_tri       = zeros(n_z_3,1);
s_tri(ide)  = N(ide)./h_QS(ide).*(1+6*abs(e(ide))./h_QS(ide)); % [N/m^2] fuer e<h/6
s_tri(~ide) = 2*N(~ide)/3./(h_QS(~ide)/2-abs(e(~ide)));        % [N/m^2] fuer e>=h/6

% Betondruckspannungen
idc=s_tri<0;
s_c=s_tri(idc)*0.75; % [N/m^2]

fprintf('\nErmittlung der Schnittgroessen und der Querschnittsparameter abgeschlossen.\n\n')

%% Ermittlung der charakteristischen Betondruckfestigkeit

fprintf('\nErmittlung der charakteristischen Betondruckfestigkeit laeuft...\n')

f_ck = zeros (n_z_3,n_z_1);
for ii = 1:n_z_3
    
    % Querschnittswerte an Nachweisstelle
    h_QS_eval = h_QS(ii);
    z_3_eval = z_3(ii);
    z_2a  = z_2_cs(ii,1);
    z_2b  = z_2_cs(ii,2);
    z_2_eval = (z_2a+h_QS_eval/(2*n_el)):h_QS_eval/n_el:z_2b;
    
    for jj = 1:n_z_1
        
        % Diskretisierung
        z_1_eval = z_1(jj);
        [zz1,zz2,zz3] = meshgrid(z_1_eval,z_2_eval,z_3_eval);
        z_eval = [zz1 zz2 zz3];
        
        % Parameter der posterior predictive Verteilung
        if strcmp(RF_dist,'N')
            data_update = data(:,2:end);
            [mu_t,lambda_t_inv,nu_t,~] = t_postpred(mu0,k0,a0,b0,R_fun,data_update,z_eval,si_eps,L_c,'mv');
            RF_dist_postpred = 't';
        else
            data_update =  [log(data(:,2)) data(:,3:end)];
            [mu_t,lambda_t_inv,nu_t,~] = t_postpred(mu0,k0,a0,b0,R_fun,data_update,z_eval,si_eps_log,L_c,'mv');
            RF_dist_postpred = 'logt';
        end
        
        % Berechnung der Parameter der Gauss'schen SA Zufallsvariablen
        [z_SA,mu_SA_Gauss,~,si_SA_Gauss] = Parameter_SA_1D(h_QS_eval,1,z_2_eval,mu_t,lambda_t_inv);
        
        % Bestimmung des charakteristischen Werts der Betondruckfestigkeit
        u_5p=tinv(0.05,nu_t); % 5%-Quantilwert der Standard-Student's t-Verteilung
        if strcmp(RF_dist_postpred,'t') % Student's t-Zufallsfeld
            f_ck(ii,jj)=mu_SA_Gauss+si_SA_Gauss.*u_5p; % 5%-Quantilwert
        else % log-Student's t-Zufallsfeld
            f_ck(ii,jj)=exp(mu_SA_Gauss+si_SA_Gauss.*u_5p); % 5%-Quantilwert
        end
        
    end
end

fprintf('\nErmittlung der charakteristischen Betondruckfestigkeit abgeschlossen.\n\n')

%% Nachweisfuehrung

fprintf('\nNachweisfuehrung laeuft...\n')

% Bemessungswert der einwirkenden Betondruckspannung
s_cd = s_c*1e-6; % [MPa]

% Bemessungswert der Betondruckfestigkeit
f_cd = f_ck*K_conc; % [MPa]

% Ueberpruefung der Bemessungsbedingung
ver = abs(s_cd./f_cd);
id_safe = ver<1;
id_nonsafe = ver>1;

z_1_ver = (z_1'*ones(1,n_z_3))';
z_3_ver = (z_3'*ones(1,n_z_1));

s_cd_plot = (s_cd*ones(1,n_z_1));
s_cd_plot = s_cd_plot(:);
f_cd_plot = f_cd(:);
ver_plot = ver(:);
id_safe_plot = id_safe(:);
id_nonsafe_plot = id_nonsafe(:);
z_1_plot = z_1_ver(:);
z_3_plot = z_3_ver(:);

names =cell(5,1);
names{1} = 'z_1';
names{2} = 'z_3';
names{3} = 's_cd';
names{4} = 'f_cd';
names{5} = 'Ausnutzung';

fprintf('\nNachweis der Betondruckspannung an %.0f von %.0f Positionen erfuellt.\n',sum(id_safe,'all'),numel(ver))
fprintf('\nPositionen an denen der Nachweis nicht erfuellt wurde:\n\n')
tab = table(z_1_plot(id_nonsafe_plot),z_3_plot(id_nonsafe_plot),s_cd_plot(id_nonsafe_plot),f_cd_plot(id_nonsafe_plot),ver_plot(id_nonsafe_plot),'VariableNames',names);
disp(tab)
fprintf('\nNachweisfuehrung abgeschlossen.\n')

%% Darstellung der Ergebnisse

% Einstellung der Figure
f1=figure;
set(f1,'PaperOrientation','landscape');
set(f1,'PaperUnits','normalized');
set(f1,'PaperPosition', [0 0 1 1]);

% Normalkraft
subplot(2,3,1);hold on
plot(N*1e-3,z_3,'r','LineWidth',1);
plot(N*1e-3,z_3,'kx','LineWidth',1);
title('Normalkraft (Druck negativ)')
xlabel('$N$ [kN/m]','Interpreter','Latex')
ylabel('$z_3$ [m]','Interpreter','Latex')
ylim([0 H])
box on; grid on; grid minor

% Biegemoment
subplot(2,3,2);hold on
plot(M*1e-3,z_3,'r','LineWidth',1);
plot(M*1e-3,z_3,'kx','LineWidth',1);
title('Biegemoment (gegen den Uhrzeigersinn)')
xlabel('$M$ [kNm/m]','Interpreter','Latex')
ylabel('$z_3$ [m]','Interpreter','Latex')
ylim([0 H])
box on; grid on; grid minor

% Querkraft
subplot(2,3,3);hold on
plot(V*1e-3,z_3,'r','LineWidth',1);
plot(V*1e-3,z_3,'kx','LineWidth',1);
title('Querkraft (\leftarrow positiv)')
xlabel('$V$ [kN/m]','Interpreter','Latex')
ylabel('$z_3$ [m]','Interpreter','Latex')
ylim([0 H])
box on; grid on; grid minor

% Exzentrizitaet
subplot(2,3,4);hold on
plot(e,z_3,'r','LineWidth',1);
plot(e,z_3,'kx','LineWidth',1);
title('Exzentrizitaet (\leftarrow positiv ab Querschnittsmitte)')
xlabel('$e$ [m]','Interpreter','Latex')
ylabel('$z_3$ [m]','Interpreter','Latex')
ylim([0 H])
box on; grid on; grid minor

% Sigma Dreieck
subplot(2,3,5);hold on
plot(s_tri(idc)*1e-6,z_3(idc),'r','LineWidth',1);
plot(s_tri(idc)*1e-6,z_3(idc),'kx','LineWidth',1);
title('\sigma_{\Delta}')
xlabel('\sigma_{\Delta} [N/mm^2]')
ylabel('$z_3$ [m]','Interpreter','Latex')
ylim([0 H])
box on; grid on; grid minor

% Sigma concrete (Compression)
subplot(2,3,6);hold on
plot(s_c*1e-6,z_3(idc),'r','LineWidth',1);
plot(s_c*1e-6,z_3(idc),'kx','LineWidth',1);
title('Bemessungswert der Betondruckspannung')
xlabel('$\sigma_{c,d}$ [MPa]','Interpreter','Latex')
ylabel('$z_3$ [m]','Interpreter','Latex')
ylim([0 H])
box on; grid on; grid minor

% Einstellung der Figure
figure; hold on
set(gcf,'renderer','Painters')
set(gcf,'PaperOrientation','landscape');

% Plot der Ergebnisse des Nachweises
scatter(z_1_plot(id_safe_plot),z_3_plot(id_safe_plot),[],'MarkerEdgeColor','k','MarkerFaceColor','g')
scatter(z_1_plot(id_nonsafe_plot),z_3_plot(id_nonsafe_plot),[],'MarkerEdgeColor','k','MarkerFaceColor','r')

axis equal tight;
xlim([0 L]);
ylim([0 H]);
box on
grid on; 
grid minor
xlabel('Position $z_1$ [m]','Interpreter','latex')
ylabel('Position $z_3$ [m]','Interpreter','latex')
title('Raeumlicher Nachweis der Betondruckfestigkeit','Interpreter','latex')
legend('Nachweis erfuellt','Nachweis nicht erfuellt','Location','SouthOutside','Orientation','horizontal')

%%% ENDE