function [N,V,M,x_sel]=SQR_QS_Oldenburg(y_sel,load_cases,ver_case)

%{
Returns the loading at any given height of the cross-section in Oldenburg
for the selected load cases

Bending moment will be applied in center of cross section at any height z
(smaller cross section height will be taken at transition points)

Bending moment positive counter-clockwise
Normal force negative for compressive force

Input:
    Compulsory
    y:     selected height of cross section, 0<=z<=9.9, from bottom to top
    cases: char array with selected loads
         G:  self weight
         W:  Water pressure (horizontal and vertical)
         P:  Water pore pressure (in joint at bottom)
         E:  Earth pressure
         T:  Traffic load
         F:  Earth friction
         A:  Reinforcing anchor
    
%}

%% Find verification case (default: characteristic values)

if nargin==2
    
    ver_case=1;
end

switch ver_case
    
    case 1
        
        gam_G=1;
        gam_W=[1 1]; % [horizontal vertical]
        gam_P=1;
        gam_E=[1 1]; % [horizontal vertical]
        gam_T=1;
        gam_F=1;
        gam_A=1;
        
    case 21
        
        gam_G=1;
        gam_W=[1.3 1.3]; % [horizontal vertical]
        gam_P=1.3;
        gam_E=[1.2 1.0]; % [horizontal vertical]
        gam_T=1.3;
        gam_F=1;
        gam_A=1;
        
    case 22
        
        gam_G=1.2;
        gam_W=[1.3 1.3]; % [horizontal vertical]
        gam_P=1.3;
        gam_E=[1.2 1.2]; % [horizontal vertical]
        gam_T=1.3;
        gam_F=1.2;
        gam_A=1;
        
    case 31
        
        gam_G=1;
        gam_W=[1.1 1.1]; % [horizontal vertical]
        gam_P=1.1;
        gam_E=[1.2 1]; % [horizontal vertical]
        gam_T=1.3;
        gam_F=1;
        gam_A=1;
        
    case 32
        
        gam_G=1;
        gam_W=[1.1 1.1]; % [horizontal vertical]
        gam_P=1.1;
        gam_E=[1.2 1.2]; % [horizontal vertical]
        gam_T=1.3;
        gam_F=1.2;
        gam_A=1;
        
end





%% Geometry

% Geometry of structure, origin at inner bottom of wall

% Bottom part
if y_sel<8
    
    x_old=[0 4.5 4.5 4 4 3 3 0.5 0.5]; % horizontal coordinates
    y_old=[0 0 4 4 6 6 8 8 1.2]; % vertical coordinates
    
    % coordinates at selected height
    if y_sel<1.2
        x_sel=[y_sel*0.5/1.2 4.5];
    elseif y_sel<4
        x_sel=[0.5 4.5];
    elseif y_sel<6
        x_sel=[0.5 4];
    else
        x_sel=[0.5 3];
    end
    
    % Effective structural geomety
    idy=find(y_old>=y_sel);
    if y_sel<1.2
        
        x_old=[x_old(idy) x_sel ];
        y_old=[y_old(idy) y_sel*ones(1,2)];
        
    else
        
        x_old=[x_old(idy) x_sel];
        y_old=[y_old(idy) y_sel*ones(1,2)];
        
    end
    
end

% Top part
x_new=[0.5 2 2 1 0.5]; % horizontal coordinates
y_new=[8 8 9.2 9.9 9.9]; % vertical coordinates

% coordinates at selected height
if y_sel>=8 &&y_sel<9.2
    x_sel=[0.5 2];
elseif y_sel>=9.2
    x_sel=[0.5 2-(y_sel-9.2)/0.7];
end

% Effective structural geomety
if y_sel>=8
    idy=find(y_new>y_sel);
    
    x_new=[x_new(idy) x_sel];
    y_new=[y_new(idy) y_sel*ones(1,2)];
    
end


%% Load from selfweight

x_mp=mean(x_sel);

idG=find(ismember(load_cases,'G'));

if idG
    
    % Structural area
    A_new=1/2*sum(x_new.*y_new([2:end,1])-y_new.*x_new([2:end,1]));
    if y_sel<8
        A_old=1/2*sum(x_old.*y_old([2:end,1])-y_old.*x_old([2:end,1]));
    end
    
    % Center of gravity for selfweight
    x_g_new=1/(6*A_new)*sum((x_new+x_new([2:end,1])).*(x_new.*y_new([2:end,1])-y_new.*x_new([2:end,1])));
    if y_sel<8
        x_g_old=1/(6*A_old)*sum((x_old+x_old([2:end,1])).*(x_old.*y_old([2:end,1])-y_old.*x_old([2:end,1])));
    end
    
    rho_new=21.5e3; % [N/m^3]
    % rho_new=22.7e3; % [N/m^3]
    N_G=-rho_new*A_new; % [N/m]
    M_G=N_G*(x_g_new-x_mp); % [Nm/m]
    
    if y_sel<8
        rho_old=21.5e3; % [N/m^3]
        % rho_old=20.9e3; % [N/m^3]
        N_G_old=-rho_old*A_old; % [N/m]
        N_G=N_G+N_G_old; % [N/m]
        M_G=M_G+N_G_old*(x_g_old-x_mp); % [Nm/m]
    end
    
    
else
    
    N_G=0;
    M_G=0;
    
end

% Multiplying partial safety factor
N_G=N_G*gam_G;
M_G=M_G*gam_G;

%% Load from Water pressure (horizontal and vertical)

% Ground water level
y_0=7.7;

idW=find(ismember(load_cases,'W'));

if idW
    
    if y_sel>=y_0
        
        N_W=0;
        V_W=0;
        M_W=0;
        
    else
        
        % horizontal pressure
        h_W=y_0-y_sel; % Pressure height [m]
        p_W=h_W*1e4; % Pressure [N/m^2]
        V_W=1/2*p_W*h_W*gam_W(1); % Force including partial safety factor [N/m]
        
        e_y=h_W/3; % lever arm [m]
        
        M_W=V_W*e_y; % Bending moment [Nm/m]
        
        % vertical pressure
        if y_sel<6
            
            N_W=-(y_0-6)*1e4*gam_W(2); % Force including partial safety factor [N/m]
            e_x=3.5-x_mp; % lever arm [m]
            M_W=M_W+N_W*e_x; % Bending moment [Nm/m]
            
            if y_sel<4
                
                N_W_2=-(y_0-4)*1e4*0.5*gam_W(2); % Force including partial safety factor [N/m]
                N_W=N_W+N_W_2; % Force [N/m]
                e_x_2=4.25-x_mp; % lever arm [m]
                
                M_W=M_W+N_W_2*e_x_2; % Bending moment [Nm/m]
                
            end
            
        else
            
            N_W=0;
            
        end
        
    end
    
else
    
    N_W=0;
    V_W=0;
    M_W=0;
    
end


%% Load from Earth pressure (horizontal and vertical)

idE=find(ismember(load_cases,'E'));

if idE
    
    
    % horizontal pressure
    % Above water
    h_E1=9.9-max(9.2,y_sel); % Pressure height [m]
    p_E1=h_E1*1.8e4; % Pressure [N/m^2]
    V_E1=1/2*p_E1*h_E1/3*gam_E(1); % Force including partial safety factor [N/m]
    
    e_y1=h_E1/3+max(9.2-y_sel,0); % lever arm [m]
    
    h_E2=9.9-max(8,y_sel); % Pressure height [m]
    p_E2=h_E2*1.8e4; % Pressure [N/m^2]
    V_E2=(p_E1+p_E2)/2*(h_E2-h_E1)/3*(9.2-y_sel>0)*gam_E(1); % Force including partial safety factor [N/m]
    
    e_y2=(h_E2-h_E1)/3*(p_E2+2*p_E1)/(p_E2+p_E1)+max(8-y_sel,0); % lever arm [m]
    
    h_E3=9.9-max(y_0,y_sel); % Pressure height [m]
    p_E3=h_E3*1.8e4; % Pressure [N/m^2]
    V_E3=(p_E2+p_E3)/2*(h_E3-h_E2)/3*(8-y_sel>0)*gam_E(1); % Force including partial safety factor [N/m]
    
    e_y3=(h_E3-h_E2)/3*(p_E3+2*p_E2)/(p_E3+p_E2)+max(7.7-y_sel,0); % lever arm [m]
    
    % Below water
    h_E4=min(max(y_0-y_sel,0),1.7); % Pressure height [m]
    p_E4=p_E3+h_E4*1e4; % Pressure [N/m^2]
    V_E4=(p_E3+p_E4)/2*h_E4/3*(7.7-y_sel>0)*gam_E(1); % Force including partial safety factor [N/m]
    
    e_y4=(h_E4)/3*(p_E4+2*p_E3)/(p_E4+p_E3)+max(6-y_sel,0); % lever arm [m]
    
    h_E5=min(max(y_0-y_sel,0),3.7); % Pressure height [m]
    p_E5=p_E3+h_E5*1e4; % Pressure [N/m^2]
    V_E5=(p_E4+p_E5)/2*(h_E5-h_E4)/3*(6-y_sel>0)*gam_E(1); % Force including partial safety factor [N/m]
    
    e_y5=(h_E5-h_E4)/3*(p_E5+2*p_E4)/(p_E5+p_E4)+max(4-y_sel,0); % lever arm [m]
    
    h_E6=min(max(y_0-y_sel,0),5.7); % Pressure height [m]
    p_E6=p_E3+h_E6*1e4; % Pressure [N/m^2]
    V_E6=(p_E5+p_E6)/2*(h_E6-h_E5)*0.279*(4-y_sel>0)*gam_E(1); % Force including partial safety factor [N/m]
    
    e_y6=(h_E6-h_E5)/3*(p_E6+2*p_E5)/(p_E6+p_E5)+max(2-y_sel,0); % lever arm [m]
    
    h_E7=min(max(y_0-y_sel,0),7.7); % Pressure height [m]
    p_E7=p_E3+h_E7*1e4; % Pressure [N/m^2]
    V_E7=(p_E6+p_E7)/2*(h_E7-h_E6)*0.251*(2-y_sel>0)*gam_E(1); % Force including partial safety factor [N/m]
    
    e_y7=(h_E7-h_E6)/3*(p_E7+2*p_E6)/(p_E7+p_E6); % lever arm [m]
    
    % Shear force from horizontal pressure
    V_E=V_E1+V_E2+V_E3+V_E4+V_E5+V_E6+V_E7; % Force including partial safety factor [N/m]
    
    % Bending moment from horizontal pressure
    M_E=V_E1*e_y1+V_E2*e_y2+V_E3*e_y3+V_E4*e_y4+V_E5*e_y5+V_E6*e_y6+V_E7*e_y7; % Bending moment [Nm/m]
    
    
    % vertical pressure
    N_E=-(9.9-max(y_sel,9.2))^2/(2*0.7)*1.8e4*gam_E(2);% Force including partial safety factor [N/m]
    e_x1=-0.25-1/3*(9.9-max(y_sel,9.2))/0.7; % lever arm [m]
    
    % Additional bending moment from vertical pressure
    M_E=M_E+N_E*e_x1; % [Nm/m]
    
    if y_sel<8
        
        N_E2=-1.9*1.8e4*gam_E(2);% Force including partial safety factor [N/m]
        e_x2=2.5-x_mp; % lever arm [m]
        
        % Additional normal force from vertical pressure
        N_E=N_E+N_E2; % [N/m]
        
        % Additional bending moment from vertical pressure
        M_E=M_E+N_E2*e_x2; % [Nm/m]
        
        if y_sel<6
            
            N_E3=-(2.2*1.8e4+1.7*1e4)*gam_E(2);% Force including partial safety factor [N/m]
            e_x3=3.5-x_mp; % lever arm [m]
            
            % Additional normal force from vertical pressure
            N_E=N_E+N_E3; % [N/m]
            
            % Additional bending moment from vertical pressure
            M_E=M_E+N_E3*e_x3; % [Nm/m]
            
            if y_sel<4
                
                N_E4=-(2.2*1.8e4+3.7*1e4)*0.5*gam_E(2);% Force including partial safety factor [N/m]
                e_x4=4.25-x_mp; % lever arm [m]
                
                % Additional normal force from vertical pressure
                N_E=N_E+N_E4; % [N/m]
                
                % Additional bending moment from vertical pressure
                M_E=M_E+N_E4*e_x4; % [Nm/m]
                
            end
            
        end
    end
    
    
else
    
    N_E=0;
    V_E=0;
    M_E=0;
    
end


%% Load from traffic (activ earth pressure, horizontal)

idT=find(ismember(load_cases,'T'));

if idT
    
    % Shear force from traffic
    V_T=min((9.9-y_sel),5.9)*2500/3*gam_T;% Force including partial safety factor [N/m]
    
    e_y=min((9.9-y_sel),5.9)/2+(y_sel<4)*(4-y_sel); % lever arm [m]
    
    % Bending moment from traffic
    M_T=V_T*e_y; % [Nm/m]
    
    if y_sel<4
        
        % Additional shear force from traffic
        V_T2=min((4-y_sel),2)*2500*0.279*gam_T;% Force including partial safety factor [N/m]
        
        e_y2=min((4-y_sel),2)/2+(y_sel<2)*(2-y_sel); % lever arm [m]
        
        V_T=V_T+V_T2; % Force [N/m]
        
        % Additional bending moment from traffic
        M_T2=V_T2*e_y2; % [Nm/m]
        
        M_T=M_T+M_T2;
        
        if y_sel<2
            
            % Additional shear force from traffic
            V_T3=(2-y_sel)*2500*0.251*gam_T;% Force including partial safety factor [N/m]
            
            e_y3=(2-y_sel)/2; % lever arm [m]
            
            V_T=V_T+V_T3; % Force [N/m]
            
            % Additional bending moment from traffic
            M_T3=V_T3*e_y3; % [Nm/m]
            
            M_T=M_T+M_T3;
            
        end
        
    end
    
else
    
    V_T=0;
    M_T=0;
    
end


%% Load from water pore pressure (uplift force from opening joint)

% pore pressure is modeled constant to the middle of the joint (from the outside) and then linearly decreasing

idP=find(ismember(load_cases,'P'));

if idP
    
    if y_sel<y_0
        
        % Constant part
        N_P1=(y_0-y_sel)*1e4*(x_sel(2)-x_mp)*gam_P;% Force including partial safety factor [N/m]
        e_P1=(x_sel(2)-x_mp)/2; % lever arm [m]
        
        % Linear part
        N_P2=(y_0-y_sel)*1e4*(x_mp-x_sel(1))/2*gam_P;% Force including partial safety factor [N/m]
        e_P2=-(x_mp-x_sel(1))/3; % lever arm [m]
        
        N_P=N_P1+N_P2; % Normal force from pore pressure [N/m]
        M_P=N_P1*e_P1+N_P2*e_P2; % Bending moment from pore pressure [Nm/m]
        
    else
        
        N_P=0;
        M_P=0;
        
    end
    
    
else
    
    N_P=0;
    M_P=0;
    
end


%% Load from earth friction (tangential force along outer wall edge)

idF=find(ismember(load_cases,'F'));

if idF & idE
    
    phi1=pi/6; % friction angle of soil
    
    if y_sel>9.2
        
        % angle of wall
        alpha=atan(1/0.7);
        
        % Tangential force from earth friction
        N_F=-V_E1/gam_E(1)*tan(2/3*phi1+alpha)*gam_F;% Force including partial safety factor [N/m]
        
        e_x=1+1/0.7*min(9.9-y_sel,0.7)/3-x_mp; % [m]
        
        M_F=N_F*e_x; % [Nm/m]
        
    elseif y_sel>8
        
        % Tangential force from earth friction
        N_F=-V_E2/gam_E(1)*tan(2/3*phi1)*gam_F; % Force including partial safety factor [N/m]
        
        e_x=x_sel(2)-x_mp; % [m]
        
        M_F=N_F*e_x; % [Nm/m]
        
    elseif y_sel>7.7
        
        % Tangential force from earth friction
        N_F=-V_E3/gam_E(1)*tan(2/3*phi1)*gam_F; % Force including partial safety factor [N/m]
        
        e_x=x_sel(2)-x_mp; % [m]
        
        M_F=N_F*e_x; % [Nm/m]
        
    elseif y_sel>6
        
        % Tangential force from earth friction
        N_F=-V_E4/gam_E(1)*tan(2/3*phi1)*gam_F; % Force including partial safety factor [N/m]
        
        e_x=x_sel(2)-x_mp; % [m]
        
        M_F=N_F*e_x; % [Nm/m]
        
    elseif y_sel>4
        
        % Tangential force from earth friction
        N_F=-V_E5/gam_E(1)*tan(2/3*phi1)*gam_F; % Force including partial safety factor [N/m]
        
        e_x=x_sel(2)-x_mp; % [m]
        
        M_F=N_F*e_x; % [Nm/m]
        
    else
        
        % Tangential force from earth friction
        N_F=-V_E6/gam_E(1)*tan(2/3*phi1)*gam_F; % Force including partial safety factor [N/m]
        
        e_x=x_sel(2)-x_mp; % [m]
        
        M_F=N_F*e_x; % [Nm/m]
        
        if y_sel<2
            
            phi2=32.5*pi/180; % friction angle of soil
            
            % Tangential force from earth friction
            N_F2=-V_E7/gam_E(1)*tan(2/3*phi2)*gam_F; % Force including partial safety factor [N/m]
            
            e_x2=x_sel(2)-x_mp; % [m]
            
            N_F=N_F+N_F2; % Force [N/m]
            M_F=M_F+N_F2*e_x2; % [Nm/m]
            
        end
        
    end
    
else
    
    N_F=0;
    M_F=0;
    
    if idF
        fprintf('\n Earth friction load only possible in combination with earth pressure');
    end
    
end


%% Load from reinforcing anchor

idA=find(ismember(load_cases,'A'));

if idA
    
    if y_sel<8
        
        % Normal force from anchor
        N_A=-221e3*gam_A;% Force including partial safety factor [N/m]
        V_A=-11.05e3*gam_A;% Force including partial safety factor [N/m]
        e_A=2.5-x_mp+(8-y_sel)/8*0.4; % [m]
        M_A=N_A*e_A;  % [Nm/m]
        
    else
        
        N_A=0;
        V_A=0;
        M_A=0;
        
    end
    
else
    
    N_A=0;
    V_A=0;
    M_A=0;
end



%% Total load on cross section

N=N_G+N_W+N_E+N_P+N_F+N_A; % Normal force [N/m]
V=V_W+V_E+V_E+V_T+V_A; % Shear force [N/m]
M=M_G+M_W+M_E+M_T+M_P+M_F+M_A; % Bending moment [Nm/m]


