function [z_SA,mu_SA,Rho_SA,si_SA]=Parameter_SA_1D(L_SA,n,zmp,mu,Cov,si,R_fun,L_c)

%{
--------------------------------------------------------------------------
Created by:                         Date:           Comment:
Sebastian Geyer                     Juli 2021         ---
Technische Universitaet Muenchen
s.geyer@tum.de
--------------------------------------------------------------------------
%}

L=sum(L_SA); % Gesamtlaenge

if nargin==8 % homogenes Zufallsfeld
    
    % Vereinfachte Berechnung nicht moeglich bei veraenderliche SA Elementlaenge
    if prod(L_SA*n/L)~=1
        
        mu     = mu*ones(length(zmp),1);
        Distwz = pdist2(zmp,zmp)/L_c;
        Cov    = si^2*R_fun(Distwz);
        type   = 'nonhom';
    else
        type = 'hom';
    end
else
    type = 'nonhom';
end

z_SA=L_SA/2;
for ii=2:n
    z_SA(ii)=z_SA(ii)+z_SA(ii-1)+L_SA(ii-1)/2; % midpoints of the averaging elements
end

if strcmp(type,'hom')
    
    L_SA = L_SA(1);
    mu_SA = mu*ones(n,1); % mean not affected in homogeneous case
    
    % Variance reduction function for calculation of covariances of SA elements
    distances  = 0:L_SA:L;
    gam        = ones(size(distances));
    
    for ii=1:size(distances,2)
        if distances(ii)>0
            f_gam=@(x_fun)(1-x_fun./distances(ii)).*R_fun(x_fun/L_c);
            gam(ii)=2/distances(ii)*integral(f_gam,0,distances(ii));
        else
            gam(ii)=1;
        end
    end
    
    % Distance matrices
    D_p    = pdist2(z_SA',z_SA');
    D0_p   = abs(D_p-L_SA);
    D2_p   = D_p+L_SA;
    
    [~,id]  = ismembertol(D_p,distances,1e-6);
    [~,id0] = ismembertol(D0_p,distances,1e-6);
    [~,id2] = ismembertol(D2_p,distances,1e-6);
    
    % Covariance matrix of the averaging elements
    Cov_SA = si^2/(2*L_SA^2)*(D0_p.^2.*gam(id0)-2*D_p.^2.*gam(id)+D2_p.^2.*gam(id2));
    
    si_SA   = sqrt(diag(Cov_SA));                          % SA standard deviation
    Rho_SA  = diag(si_SA.^(-1))*Cov_SA*diag(si_SA.^(-1));  % SA correlation matrix
    
else
    
    dz    = zmp(2)-zmp(1);
    mu_SA = ones(n,1);
    ne    = length(zmp);
    ne_SA = round(L_SA/L*ne,2);
    count = ones(1,n);
    
    % SA mean
    for ii=1:n
        mu_SA(ii) = mean(mu(count(ii):count(ii)+ne_SA(ii)-1));
        if ii<n
            count(ii+1) = count(ii)+ne_SA(ii);
        end
    end
    
    % Covariance matrix of the spatial average
    Cov_SA = ones(n);
    
    for ii=1:n
        
        % First and last discretization point of ii-th SA element
        i1a = count(ii);
        i1b = count(ii)+ne_SA(ii)-1;
        
        for jj=1:n
            
            % Omit covarianve calculation if ii and jj are the same -> evaluate SA variance directly by evaluating variance reduction function
            if ii == jj
                Cov_SA(ii,jj) = dz^2/L_SA(ii)^2*sum(sum(Cov(i1a:i1b,i1a:i1b)));
                
                % Take advantage of symmetry of covariance matrix
            elseif jj<ii
                Cov_SA(ii,jj) = Cov_SA(jj,ii);
                
                % Solve numerical integrals for elements ii and jj
            else
                % First and last discretization point of jj-th SA element
                i2a = count(jj);
                i2b = count(jj)+ne_SA(jj)-1;
                
                % Integration lengths
                c0 = (i2a-i1b-1)/(i2a-i1b); % Distance between end of ii and beginning of jj
                c1 = (i2a-i1a-1)/(i2a-i1a); % Distance between beginning of ii and beginning of jj
                c2 = (i2b-i1a-1)/(i2b-i1a); % Distance between beginning of ii and end of jj
                c3 = (i2b-i1b-1)/(i2b-i1b); % Distance between end of ii and end of jj
                
                % Evaluate variance reduction function for lengths c0, c1, c2, c3
                D0 = c0*dz^2*sum(sum(Cov(min(i1b,i2a):max(i1b,i2a),min(i1b,i2a):max(i1b,i2a))));
                D1 = c1*dz^2*sum(sum(Cov(min(i1a,i2a):max(i1a,i2a),min(i1a,i2a):max(i1a,i2a))));
                D2 = c2*dz^2*sum(sum(Cov(min(i1a,i2b):max(i1a,i2b),min(i1a,i2b):max(i1a,i2b))));
                D3 = c3*dz^2*sum(sum(Cov(min(i1b,i2b):max(i1b,i2b),min(i1b,i2b):max(i1b,i2b))));
                
                % Determine Covariance of element ii and jj from the variances of c0, c1, c2 and c3
                Cov_SA(ii,jj) = 1/(2*L_SA(ii)*L_SA(jj))*(D0-D1+D2-D3);
                
            end
            
        end
        
    end
    
    si_SA   = sqrt(diag(Cov_SA));                                    % SA standard deviation
    Rho_SA  = diag(si_SA.^(-1))*Cov_SA*diag(si_SA.^(-1));  % SA correlation matrix
    
end

end