%% Spatial averaging fuer eindimensionale Zufallsfelder

clear all; close all; clc;

%{
--------------------------------------------------------------------------
Created by:                         Date:           Comment:
Sebastian Geyer                     Juli 2021         ---
Technische Universitaet Muenchen
s.geyer@tum.de
--------------------------------------------------------------------------
%}

%% Wahl der Art des Zufallsfelds

% Homogenitaet des Zufallsfelds
RF_hom = false; % true: homogen; false: nicht-homogen

if RF_hom     % Definition des homogenen Zufallsfelds
    
    % Geometrie
    obj_RF.L = 50;   % Laenge des zu untersuchenden Bereichs [m]
    L_el     = 0.02; % Laenge der Diskretisierungselemente [m]
    
    % Verteilungstyp und -momente
    obj_RF.RF_dist = 'N'; % 'N': Normalverteilung, 'logN': Lognormalverteilung
    obj_RF.mu      = 20;  % Mittelwert
    obj_RF.si      = 5 ;  % Standardabweichung
    
    % Korrelationsmodell
    nu         = 2.5; % Smoothness-Parameter des Matern-Modells (nur fuer Werte von nu = N+0.5 implementiert (N positiv und ganzzahlig), zus.: nu = inf (square-exp. Modell))
    obj_RF.L_c = 5;   % Korrelationslaenge [m]
    g_mic      = 0.1; % Anteil der Mikrokorrelation (0 <= g_mic <= 1)
    
else % Laden des nicht-homogenen Zufallsfelds aus externer Datei
    load('obj_RF')
end

%% Modellparameter

n_SA = 10;             % Anzahl der SA Elemente
L_SA = obj_RF.L/n_SA;  % Laenge der SA Elemente [m] (Skalar: Alle Elemente gleich lang; Vektor: Jedes Element mit spezifischer Laenge)

% Darstellung der Ergebnisse
plot_res  = true; % (plot_res = true: Ergebnisse darstellen; plot_res = false: Ergebnisse nicht darstellen)
plot_real = 3;    % (Anzahl der Realisierungen, die illustriert werden sollen)

% Randpunkte der SA Elemente [m]
if length(L_SA)==1
    z_bounds_SA = 0:obj_RF.L/n_SA:obj_RF.L;
else
    z_bounds_SA = zeros(n_SA+1,1);
    for ii =2:n_SA+1
        z_bounds_SA(ii)=z_bounds_SA(ii-1)+L_SA(ii-1);
    end
end

if RF_hom
    
    % Parameter der (zugrundeliegenden) Normalverteilung
    if strcmp(obj_RF.RF_dist,'logN')
        mu_n=log(obj_RF.mu^2./sqrt(obj_RF.si^2+obj_RF.mu^2));
        si_n=sqrt(log(obj_RF.si^2/obj_RF.mu^2+1));
    else
        mu_n = obj_RF.mu;
        si_n = obj_RF.si;
    end
    
    % Diskretisierung
    obj_RF.z_eval = (L_el/2:L_el:obj_RF.L)';
    
    % Korrelationsfunktion
    R_fun_meso = Matern_corr(nu);
    obj_RF.R_fun = @(Dw)g_mic*(Dw==0)+(1-g_mic)*R_fun_meso(Dw);
    
    % Definition der Funktion zur Generierung von Stichproben
    D_eval = pdist2(obj_RF.z_eval,obj_RF.z_eval);
    Dweval = D_eval/obj_RF.L_c;
    R_eval = obj_RF.R_fun(Dweval);
    if strcmp(obj_RF.RF_dist,'N') % Gauss'sches Zufallsfeld
        obj_RF.f_rand = @(n_rand)mvnrnd(mu_n*ones(1,length(obj_RF.z_eval)),si_n^2*R_eval,n_rand)';
    else % lognormales Zufallsfeld
        obj_RF.f_rand = @(n_rand)exp(mvnrnd(mu_n*ones(1,length(obj_RF.z_eval)),si_n^2*R_eval,n_rand)');
    end
    
end

zmp  = obj_RF.z_eval;  % Sttzstellen fr die numerische Integration [m]
n_el = length(zmp); % Anzahl der Sttzstellen fr die numerische Integration

%% Ueberpruefung der Eingabe

if  mod(round(L_SA/obj_RF.L*n_el,2),1)
    error('Durchfuehrung der SA Approximation nicht moeglich, ganzzahlige Anzahl an Stuetzstellen je SA Element erforderlich')
elseif length(L_SA)~=n_SA && length(L_SA)>1
    error('Anzahl der SA Elemente ist nicht konsistent mit dem Vektor mit den zugehrigen Laengen der Elemente')
elseif sum(L_SA)~=obj_RF.L && length(L_SA)>1
    error('Summe der Einzellaengen der SA Elemente ergibt nicht die Gesamtlaenge')
elseif ~islogical(plot_res)
    error('Art der Ergebnisdarstellung nicht korrekt definiert, bitte ueberpruefen')
else
    if ~islogical(RF_hom)
        error('Homogenitaet des Zufallsfelds muss true oder false sein')
    elseif ~RF_hom
        if ~strcmp(obj_RF.type,'mv')
            error('Eingabedatei nicht korrekt definiert, multivariate Verteilung erforderlich')
        end
    elseif RF_hom
        if ~strcmp(obj_RF.RF_dist,'N') && ~strcmp(obj_RF.RF_dist,'logN')
            error('Verteilungstyp nicht korrekt definiert, bitte ueberpruefen')
        elseif obj_RF.si<0
            error('Standardabweichung muss nicht-negativ sein')
        elseif mod(nu-0.5,1)~=0
            error('Smoothness-Parameter nicht korrekt definiert, bitte ueberpruefen')
        elseif obj_RF.L_c <0
            error('Korrelationslaenge nicht korrekt definiert, bitte ueberpruefen')
        elseif g_mic <0 || g_mic >1
            error('Anteil der Mikrokorrelation nicht korrekt definiert, bitte ueberpruefen')
        end
    end
end

fprintf('\nEingabeparameter in Ordnung\n\n')

% Probabilistische Eingangsparameter
dist_type = obj_RF.RF_dist;   % Verteilungstyp des Zufallsfelds (N: Gaussian; logN: lognormal; t: Student's t; logt: log-Student's t)

%% Spatial averaging

if length(L_SA)==1
    L_SA = L_SA*ones(1,n_SA);
end

fprintf('\nVerfahren zur Bestimmung der Parameter der SA-Zufallsvariablen laeuft...\n')

if RF_hom % SA fuer homogenes Zufallsfeld
    
    % Berechnung der Parameter der Gauss'schen SA Zufallsvariablen
    [z_SA,mu_SA_Gauss,Rho_SA_Gauss,si_SA_Gauss] = Parameter_SA_1D(L_SA,n_SA,zmp,mu_n,[],si_n,obj_RF.R_fun,obj_RF.L_c);
    Sigma_SA_Gauss = diag(si_SA_Gauss)*Rho_SA_Gauss*diag(si_SA_Gauss);
    
else % SA fuer nicht-homogenes Zufallsfeld
    
    if strcmp(dist_type,'N') || strcmp(dist_type,'logN')
        
        % Berechnung der Parameter der Gauss'schen SA Zufallsvariablen
        [z_SA,mu_SA_Gauss,Rho_SA_Gauss,si_SA_Gauss] = Parameter_SA_1D(L_SA,n_SA,zmp,obj_RF.mu_n,obj_RF.Sigma_n);
        Sigma_SA_Gauss = diag(si_SA_Gauss)*Rho_SA_Gauss*diag(si_SA_Gauss);
        
    else
        
        % Berechnung der Parameter der Gauss'schen SA Zufallsvariablen
        [z_SA,mu_SA_Gauss,Rho_SA_Gauss,si_SA_Gauss] = Parameter_SA_1D(L_SA,n_SA,zmp,obj_RF.mu_t,obj_RF.lambda_t_inv);
        Sigma_SA_Gauss = diag(si_SA_Gauss)*Rho_SA_Gauss*diag(si_SA_Gauss);
        
    end
end

fprintf('\nBestimmung der Parameter der SA-Zufallsvariablen erfolgreich\n\n')

% Zustzliche Eigenschaften des SA Objekts
obj_SA.dist = obj_RF.RF_dist;                  % Verteilungstyp der SA Variablen
obj_SA.z_mp = z_SA';                            % Mittelpunkte der SA Elemente

if strcmp(dist_type,'N') || strcmp(dist_type,'logN') % Gauss'sches oder lognormales Zufallsfeld
    
    % Definition der Funktion zur Generierung von Stichproben
    if strcmp(dist_type,'N') % Gauss'sches Zufallsfeld
        f_rand = @(n_rand)mvnrnd(mu_SA_Gauss',Sigma_SA_Gauss,n_rand)';
    else % lognormales Zufallsfeld
        f_rand = @(n_rand)exp(mvnrnd(mu_SA_Gauss',Sigma_SA_Gauss,n_rand)');
    end
    
    % Berechnung von Quantilwerten der SA Zufallsvariablen
    u_5p=norminv(0.05); % 5%-Quantilwert der Standardnormalverteilung
    
    if strcmp(dist_type,'N') % Gauss'sches Zufallsfeld
        x_5p=mu_SA_Gauss+si_SA_Gauss.*u_5p; % 5%-Quantilwert
        x_95p=mu_SA_Gauss-si_SA_Gauss.*u_5p; % 95%-Quantilwert
        x_m=mu_SA_Gauss; % Median
    else % lognormales Zufallsfeld
        x_5p=exp(mu_SA_Gauss+si_SA_Gauss.*u_5p); % 5%-Quantilwert
        x_95p=exp(mu_SA_Gauss-si_SA_Gauss.*u_5p); % 95%-Quantilwert
        x_m=exp(mu_SA_Gauss); % Median
    end
    
    % Abspeichern der Ergebnisse
    if strcmp(dist_type,'logN')
        [obj_SA.mu,var] = lognstat(mu_SA_Gauss,si_SA_Gauss); % Mittelwert und Varianz der lognormalen SA Variablen
        obj_SA.si = sqrt(var);                 % Standardabweichung der lognormalen SA Variablen
    end
    obj_SA.mu_n = mu_SA_Gauss;       % Mittelwerte der (zugrundeliegenden) Gauss'schen SA Variablen
    obj_SA.Sigma_n = Sigma_SA_Gauss; % Kovarianz der (zugrundeliegenden) Gauss'schen SA Variablen
    
else % Student's t- oder log-Student's t-Zufallsfeld
    
    % Definition der Funktion zur Generierung von Stichproben
    if strcmp(dist_type,'t') % Student's t-Zufallsfeld
        f_rand = @(n_rand)(ones(n_SA,1)*(obj_RF.nu_t./chi2rnd(obj_RF.nu_t,1,n_rand)).^(1/2)).*mvnrnd(zeros(1,n_SA),Sigma_SA_Gauss,n_rand)'+mu_SA_Gauss;
    else % log-Student's t-Zufallsfeld
        f_rand = @(n_rand)exp((ones(n_SA,1)*(obj_RF.nu_t./chi2rnd(obj_RF.nu_t,1,n_rand)).^(1/2)).*mvnrnd(zeros(1,n_SA),Sigma_SA_Gauss,n_rand)'+mu_SA_Gauss);
    end
    
    % Berechnung von Quantilwerten der SA Zufallsvariablen
    Sigma_t_SA = Sigma_SA_Gauss*obj_RF.nu_t/(obj_RF.nu_t-2); % Kovarianzmatrix der (zugrundeliegenden) Student's t-Verteilung
    si_t_SA = sqrt(diag(Sigma_t_SA));
    u_5p=tinv(0.05,obj_RF.nu_t); % 5%-Quantilwert der Standard-Student's t-Verteilung
    
    if strcmp(dist_type,'t') % Student's t-Zufallsfeld
        x_5p=mu_SA_Gauss+sqrt((obj_RF.nu_t-2)/(obj_RF.nu_t))*si_t_SA.*u_5p; % 5%-Quantilwert
        x_95p=mu_SA_Gauss-sqrt((obj_RF.nu_t-2)/(obj_RF.nu_t))*si_t_SA.*u_5p; % 95%-Quantilwert
        x_m=mu_SA_Gauss; % Median
    else % log-Student's t-Zufallsfeld
        x_5p=exp(mu_SA_Gauss+sqrt((obj_RF.nu_t-2)/(obj_RF.nu_t))*si_t_SA.*u_5p); % 5%-Quantilwert
        x_95p=exp(mu_SA_Gauss-sqrt((obj_RF.nu_t-2)/(obj_RF.nu_t))*si_t_SA.*u_5p); % 95%-Quantilwert
        x_m=exp(mu_SA_Gauss); % Median
    end
    
    % Abspeichern der Ergebnisse
    obj_SA.mu_t = mu_SA_Gauss;                     % Lageparameter der (zugrundeliegenden) Student's t-SA Variablen
    obj_SA.lambda_t = inv(Sigma_SA_Gauss);         % Skalenparameter der (zugrundeliegenden) Student's t-SA Variablen
    obj_SA.nu_t = obj_RF.nu_t;                     % Freiheitsgrade der (zugrundeliegenden) Student's t-SA Variablen
    
end

obj_SA.f_rand = f_rand;                        % Funktion zur Erzeugung von Stichproben
obj_SA.x_m = x_m;                              % Median der SA Variablen
obj_SA.x_5p = x_5p;                            % 5%-Quantilwert der SA Variablen
obj_SA.x_95p = x_95p;                          % 95%-Quantilwert der SA Variablen

% Abspeichern der Ergebnisse
save('obj_SA','obj_SA');

fprintf('\nErgebnis in Variable ''obj_SA.mat'' abgespeichert\n')

%% Darstellung der Ergebnisse

if plot_res
    
    z_0 = obj_SA.z_mp'-L_SA/2;
    z_1 = obj_SA.z_mp'+L_SA/2;
    z_01 = [z_0;z_1];
    
    % Einstellung der Figure
    figure; hold on
    set(gcf,'renderer','Painters')
    set(gcf,'PaperOrientation','landscape');
    
    % Plot der Ergebnisse
    for ii = 1:n_SA
        plot([z_0(ii) z_1(ii)],x_5p(ii)*ones(1,2),'Color','k','LineWidth',1)
        plot([z_0(ii) z_1(ii)],x_m(ii)*ones(1,2),'Color','b','LineWidth',1)
        plot([z_0(ii) z_1(ii)],x_95p(ii)*ones(1,2),'Color','k','LineWidth',1)
        if ii<n_SA
            plot([z_1(ii) z_0(ii+1)],[x_5p(ii) x_5p(ii+1)],'Color','k','LineStyle',':','LineWidth',1)
            plot([z_1(ii) z_0(ii+1)],[x_m(ii) x_m(ii+1)],'Color','b','LineStyle',':','LineWidth',1)
            plot([z_1(ii) z_0(ii+1)],[x_95p(ii) x_95p(ii+1)],'Color','k','LineStyle',':','LineWidth',1)
        end
    end
    
    plot(z_01(:),kron(x_m,ones(2,1)),'ko','LineWidth',1,'MarkerSize',3)
    plot(z_01(:),kron(x_5p,ones(2,1)),'ko','LineWidth',1,'MarkerSize',3)
    plot(z_01(:),kron(x_95p,ones(2,1)),'ko','LineWidth',1,'MarkerSize',3)
    filled_area=fill([z_01(:);flipud(z_01(:))],[kron(x_5p,ones(2,1));flipud(kron(x_95p,ones(2,1)))],[0.5 0.5 0.5],'LineStyle','none');
    set(filled_area,'facealpha',.35);
    legend('90%-Intervallgrenzen','Median','Location','SouthOutside','Orientation','horizontal')
    set(gca,'children',flipud(get(gca,'children')))
    box on;
    grid on;
    xlabel('Position [m]')
    ylabel('Druckfestigkeit [MPa]')
    title('Medianwerte und 90\%-Intervalle der SA-Zufallsvariablen','Interpreter','latex')
    
end

if plot_real>0
    
    z_0 = obj_SA.z_mp'-L_SA/2;
    z_1 = obj_SA.z_mp'+L_SA/2;
    z_01 = [z_0;z_1];
    
    % Einstellung der Figure
    figure; hold on
    set(gcf,'renderer','Painters')
    set(gcf,'PaperOrientation','landscape');
    
    x_real = obj_SA.f_rand(plot_real);
    
    cmap=lines(7);
    % Plot der Ergebnisse
    for jj = 1:plot_real
        for ii = 1:n_SA
            
            plot([z_0(ii) z_1(ii)],x_real(ii,jj)*ones(1,2),'Color',cmap(mod(jj,7),:),'LineWidth',1)
            if ii<n_SA
                plot([z_1(ii) z_0(ii+1)],[x_real(ii,jj) x_real(ii+1,jj)],'Color','k','LineStyle',':','LineWidth',1)
            end
        end
        plot(z_01(:),kron(x_real(:,jj),ones(2,1)),'ko','LineWidth',1,'MarkerSize',3)
        
    end
    box on;
    grid on;
    xlabel('Position [m]')
    ylabel('Druckfestigkeit [MPa]')
    title('Stichproben der SA-Zufallsvariablen','Interpreter','latex')
    
    
end

%%% ENDE