%% Spatial averaging fuer zweidimensionale Zufallsfelder

clear all; close all; clc;

%{
--------------------------------------------------------------------------
Created by:                         Date:           Comment:
Sebastian Geyer                     Juli 2021         ---
Technische Universitaet Muenchen
s.geyer@tum.de
--------------------------------------------------------------------------
%}

%% Wahl der Art des Zufallsfelds

% Homogenitaet des Zufallsfelds
obj_RF.RF_hom = false; % true: homogen; false: nicht-homogen (Bayes'sches Update)

% Geometrie
obj_RF.L = [20 20];  % Laenge des zu untersuchenden Bereichs je Dimension [m]
n_Gp     = 50;       % Anzahl der Stuetzstellen je Dimension und Element fuer die Integration

% Verteilungstyp des Zufallsfelds (a-priori Modell bei Bayes'schem Update)
obj_RF.RF_dist = 'logN'; % 'N': Normalverteilung, 'logN': Lognormalverteilung

% Korrelationsmodell (a-priori Modell bei Bayes'schem Update)
nu    = 2.5;     % Smoothness-Parameter des Matern-Modells (nur fuer Werte von nu = N+0.5 implementiert (N positiv und ganzzahlig), zus.: nu = inf (square-exp. Modell))
L_c   = [5 5];   % Korrelationslaenge [m]
g_mic = 0.25;   % Anteil der Mikrokorrelation (0 <= g_mic <= 1)

if obj_RF.RF_hom  % Definition des homogenen Zufallsfelds
    
    mu = 20 ; % Mittelwert
    si = 5 ;  % Standardabweichung
    
else
     
    % A-priori-Parameter der Normal-Gamma-Verteilung (Noninformative a-priori-Parameter: mu0 = 0, k0 = 0, a0 = -1/2, b0 = 0)
    obj_RF.mu0 = 0;    % Mittelwert der Normalverteilung
    obj_RF.k0  = 0;    % Skalierungsfaktor fuer precision der Normalverteilung
    obj_RF.a0  = -1/2; % Formparameter der Gammaverteilung
    obj_RF.b0  = 0;    % Inverser Skalenparameter der Gammaverteilung
    
    % Parameter der Messungenauigkeit
    switch obj_RF.RF_dist
        case 'N'    % (normal, additiv)
            si_eps = 0; % (si_eps = 0: Keine Messungenauigkeit)
        case 'logN' % (lognormal, multiplikativ)
            CV_eps     = 0; % Variationskoeffizient (CV_eps = 0: Keine Messungenauigkeit)
            si_eps_log = sqrt(log(1+CV_eps^2)); % Standardabweichung der zugrundeliegenden Normalverteilung (si_eps*si_X)
    end
    
    % Laden der Daten aus externer Datei
    data = readmatrix('Daten_dummy_test_2D.xlsx','Sheet','Daten_DF','Range','A2:D11');
    
end

%% Weitere Modellparameter

n_SA = [3 2]; % Anzahl der SA Elemente je Dimension (Zeilenvektor 1 x 2)

% Darstellung der Ergebnisse
plot_res  = true; % (plot_res = true: Ergebnisse darstellen; plot_res = false: Ergebnisse nicht darstellen)
plot_real = 3;    % (Anzahl der Realisierungen, die illustriert werden sollen)

%% Vorberechnungen
L_SA_1 = obj_RF.L(1)/n_SA(1);  % Laenge der SA Elemente in Richtung z_1 [m] (Skalar: alle Elemente gleich lang)
L_SA_2 = obj_RF.L(2)/n_SA(2);  % Laenge der SA Elemente in Richtung z_2 [m] (Skalar: alle Elemente gleich lang)

% Randpunkte der SA Elemente [m]
z1_bounds_SA = 0:obj_RF.L(1)/n_SA(1):obj_RF.L(1);
z2_bounds_SA = 0:obj_RF.L(2)/n_SA(2):obj_RF.L(2);
[zz1_bounds_SA,zz2_bounds_SA] = meshgrid(z1_bounds_SA,z2_bounds_SA');

% Korrelationsfunktion (a-priori Modell bei Bayes'schem Update)
R_fun_meso   = Matern_corr(nu);
obj_RF.R_fun = @(Dw)g_mic*(Dw==0)+(1-g_mic)*R_fun_meso(Dw);
obj_RF.L_c   = L_c;

if obj_RF.RF_hom
    
    % Parameter der (zugrundeliegenden) Normalverteilung
    if strcmp(obj_RF.RF_dist,'logN')
        obj_RF.mu_n=log(mu^2./sqrt(si^2+mu^2));
        obj_RF.si_n=sqrt(log(si^2/mu^2+1));
    else
        obj_RF.mu_n = mu;
        obj_RF.si_n = si;
    end
    
end

% Log-Transformation der Daten, falls ntig
if ~obj_RF.RF_hom
    switch obj_RF.RF_dist
        case 'N'
            obj_RF.data_update   = data(:,2:4);
            obj_RF.si_eps_update = si_eps;
        case 'logN'
            obj_RF.data_update =  [log(data(:,2)) data(:,3:4)];
            obj_RF.si_eps_update = si_eps_log;
    end
end

%% Ueberpruefung der Eingabe

if size(obj_RF.L,2)~=2
    error('Laenge des zu untersuchenden Bereichs je Dimension nicht korrekt definiert, bitte ueberpruefen')
elseif size(n_SA,2)~=2
    error('Anzahl der SA Elemente je Dimension nicht korrekt definiert, bitte ueberpruefen')
elseif ~strcmp(obj_RF.RF_dist,'N') && ~strcmp(obj_RF.RF_dist,'logN')
    error('Verteilungstyp nicht korrekt definiert, bitte ueberpruefen')
elseif mod(nu-0.5,1)~=0
    error('Smoothness-Parameter nicht korrekt definiert, bitte ueberpruefen')
elseif min(L_c) <0
    error('Korrelationslaenge nicht korrekt definiert, bitte ueberpruefen')
elseif g_mic <0 || g_mic >1
    error('Anteil der Mikrokorrelation nicht korrekt definiert, bitte ueberpruefen')
elseif ~islogical(plot_res)
    error('Art der Ergebnisdarstellung nicht korrekt definiert, bitte ueberpruefen')
else
    if ~islogical(obj_RF.RF_hom)
        error('Homogenitaet des Zufallsfelds muss true oder false sein')
    elseif ~obj_RF.RF_hom
        if size(data,2)~=4
            error('Datei mit Messwerten nicht korrekt definiert, bitte ueberpruefen')
        end
    elseif obj_RF.RF_hom
        if si<0
            error('Standardabweichung muss nicht-negativ sein')
        end
    end
end

fprintf('\nEingabeparameter in Ordnung\n\n')

%% Spatial averaging

fprintf('\nVerfahren zur Bestimmung der Parameter der SA-Zufallsvariablen laeuft...\n')

n_SA_tot = prod(n_SA);
count_el     = 0;
count_cov    = 0;
count_cov_el = n_SA_tot-1;
n_cov_offd   = n_SA_tot/2*(n_SA_tot-1);
mu_SA_Gauss = zeros(n_SA_tot,1);
si_SA_Gauss = zeros(n_SA_tot,1);
Sigma_SA_Gauss = zeros(n_SA_tot);
z1_SA_m_vec  = zeros(n_SA_tot,1);
z2_SA_m_vec  = zeros(n_SA_tot,1);


% SA fuer Zufallsfeld
tic;
for ii=1:n_SA(1)
    for jj=1:n_SA(2)
        [mu_SA_Gauss((ii-1)*n_SA(2)+jj),si_SA_Gauss((ii-1)*n_SA(2)+jj),Sigma_SA_Gauss,z1_SA_m_vec((ii-1)*n_SA(2)+jj),z2_SA_m_vec((ii-1)*n_SA(2)+jj)] = Parameter_SA_2D(ii,jj,zz1_bounds_SA,zz2_bounds_SA,obj_RF,Sigma_SA_Gauss,n_Gp);
        count_cov = count_cov +count_cov_el;
        count_el  = count_el+1;
        t1 = toc;
        
        fprintf('\n%g von %g Termen des SA-Mittelwertsvektors berechnet',count_el,n_SA_tot);
        fprintf('\n%g von %g Diagonaltermen der SA-Kovarianzmatrix berechnet',count_el,n_SA_tot);
        fprintf('\n%g von %g nicht-diagonalen Termen der SA-Kovarianzmatrix berechnet',count_cov,n_cov_offd);     
        fprintf('\nBentigte Zeit: %.3f min \n',t1/60);
        count_cov_el = count_cov_el-1;
        
    end
end

fprintf('\nBestimmung der Parameter der SA-Zufallsvariablen erfolgreich\n')

% Verteilungstyp der SA Variablen
if obj_RF.RF_hom
    obj_SA.dist = obj_RF.RF_dist;
else
    if strcmp(obj_RF.RF_dist,'N')
        obj_SA.dist = 't';
    else
        obj_SA.dist = 'logt';
    end
    nu_t = 2*obj_RF.a0+size(obj_RF.data_update,1); % Anzahl der Freiheitsgrade der (zugrundeliegenden) Student's t-Verteilung
end

% Mittelpunkte der SA Elemente
obj_SA.z1_mp = z1_SA_m_vec;
obj_SA.z2_mp = z2_SA_m_vec;

if strcmp(obj_SA.dist,'N') || strcmp(obj_SA.dist,'logN') % Gauss'sches oder lognormales Zufallsfeld
    
    % Definition der Funktion zur Generierung von Stichproben
    if strcmp(obj_SA.dist,'N') % Gauss'sches Zufallsfeld
        f_rand = @(n_rand)mvnrnd(mu_SA_Gauss',Sigma_SA_Gauss,n_rand)';
    else % lognormales Zufallsfeld
        f_rand = @(n_rand)exp(mvnrnd(mu_SA_Gauss',Sigma_SA_Gauss,n_rand)');
    end
    
    % Berechnung von Quantilwerten der SA Zufallsvariablen
    u_5p=norminv(0.05); % 5%-Quantilwert der Standardnormalverteilung
    
    if strcmp(obj_SA.dist,'N') % Gauss'sches Zufallsfeld
        x_5p=mu_SA_Gauss+si_SA_Gauss.*u_5p; % 5%-Quantilwert
        x_95p=mu_SA_Gauss-si_SA_Gauss.*u_5p; % 95%-Quantilwert
        x_m=mu_SA_Gauss; % Median
    else % lognormales Zufallsfeld
        x_5p=exp(mu_SA_Gauss+si_SA_Gauss.*u_5p); % 5%-Quantilwert
        x_95p=exp(mu_SA_Gauss-si_SA_Gauss.*u_5p); % 95%-Quantilwert
        x_m=exp(mu_SA_Gauss); % Median
    end
    
    % Abspeichern der Ergebnisse
    if strcmp(obj_SA.dist,'logN')
        [obj_SA.mu,var] = lognstat(mu_SA_Gauss,si_SA_Gauss); % Mittelwert und Varianz der lognormalen SA Variablen
        obj_SA.si = sqrt(var);                 % Standardabweichung der lognormalen SA Variablen
    end
    obj_SA.mu_n = mu_SA_Gauss;       % Mittelwerte der (zugrundeliegenden) Gauss'schen SA Variablen
    obj_SA.Sigma_n = Sigma_SA_Gauss; % Kovarianz der (zugrundeliegenden) Gauss'schen SA Variablen
    
else % Student's t- oder log-Student's t-Zufallsfeld
    
    % Definition der Funktion zur Generierung von Stichproben
    if strcmp(obj_SA.dist,'t') % Student's t-Zufallsfeld
        f_rand = @(n_rand)(ones(n_SA_tot,1)*(nu_t./chi2rnd(nu_t,1,n_rand)).^(1/2)).*mvnrnd(zeros(1,n_SA_tot),Sigma_SA_Gauss,n_rand)'+mu_SA_Gauss;
    else % log-Student's t-Zufallsfeld
        f_rand = @(n_rand)exp((ones(n_SA_tot,1)*(nu_t./chi2rnd(nu_t,1,n_rand)).^(1/2)).*mvnrnd(zeros(1,n_SA_tot),Sigma_SA_Gauss,n_rand)'+mu_SA_Gauss);
    end
    
    % Berechnung von Quantilwerten der SA Zufallsvariablen
    Sigma_t_SA = Sigma_SA_Gauss*nu_t/(nu_t-2); % Kovarianzmatrix der (zugrundeliegenden) Student's t-Verteilung
    si_t_SA = sqrt(diag(Sigma_t_SA));
    u_5p=tinv(0.05,nu_t); % 5%-Quantilwert der Standard-Student's t-Verteilung
    
    if strcmp(obj_SA.dist,'t') % Student's t-Zufallsfeld
        x_5p=mu_SA_Gauss+sqrt((nu_t-2)/(nu_t))*si_t_SA.*u_5p; % 5%-Quantilwert
        x_95p=mu_SA_Gauss-sqrt((nu_t-2)/(nu_t))*si_t_SA.*u_5p; % 95%-Quantilwert
        x_m=mu_SA_Gauss; % Median
    else % log-Student's t-Zufallsfeld
        x_5p=exp(mu_SA_Gauss+sqrt((nu_t-2)/(nu_t))*si_t_SA.*u_5p); % 5%-Quantilwert
        x_95p=exp(mu_SA_Gauss-sqrt((nu_t-2)/(nu_t))*si_t_SA.*u_5p); % 95%-Quantilwert
        x_m=exp(mu_SA_Gauss); % Median
    end
    
    % Abspeichern der Ergebnisse
    obj_SA.mu_t = mu_SA_Gauss;                     % Lageparameter der (zugrundeliegenden) Student's t-SA Variablen
    obj_SA.lambda_t = inv(Sigma_SA_Gauss);         % Skalenparameter der (zugrundeliegenden) Student's t-SA Variablen
    obj_SA.nu_t = nu_t;                     % Freiheitsgrade der (zugrundeliegenden) Student's t-SA Variablen
    
end

obj_SA.f_rand = f_rand;                        % Funktion zur Erzeugung von Stichproben
obj_SA.x_m = x_m;                              % Median der SA Variablen
obj_SA.x_5p = x_5p;                            % 5%-Quantilwert der SA Variablen
obj_SA.x_95p = x_95p;                          % 95%-Quantilwert der SA Variablen

% Abspeichern der Ergebnisse
save('obj_SA','obj_SA');

fprintf('\n\nErgebnis in Variable ''obj_SA.mat'' abgespeichert\n')

%% Darstellung der Ergebnisse

if plot_res
    
    x_min = min(obj_SA.x_5p);
    x_max = max(obj_SA.x_95p);
    c_range_par = (x_max-x_min);
    cmap_par = flipud(hot);
    
    c_id_m   = max(1,round((obj_SA.x_m-x_min)/c_range_par*256));
    c_id_5p  = max(1,round((obj_SA.x_5p-x_min)/c_range_par*256));
    c_id_95p = max(1,round((obj_SA.x_95p-x_min)/c_range_par*256));
    
    % Einstellung der Figure
    figure; hold on;
    set(gcf,'renderer','Painters')
    set(gcf,'color','w');
    
    % Plot des Medians
    for ii=1:n_SA_tot
        color=cmap_par(c_id_m(ii),:);
        rectangle('Position',[obj_SA.z1_mp(ii)-L_SA_1/2,obj_SA.z2_mp(ii)-L_SA_2/2,L_SA_1,L_SA_2],'FaceColor',color,'EdgeColor','c')
    end
    colormap(cmap_par)
    colorbar
    caxis([x_min x_max])
    axis equal tight;
    box on
    xlabel('$z_1$ [m]','Interpreter','latex')
    ylabel('$z_2$ [m]','Interpreter','latex')
    title('Median der SA-Zufallsvariablen [MPa]','Interpreter','latex')
    
    % Einstellung der Figure
    figure; hold on;
    set(gcf,'renderer','Painters')
    set(gcf,'color','w');
    
    % Plot des 5%-Quantils
    for ii=1:n_SA_tot
        color=cmap_par(c_id_5p(ii),:);
        rectangle('Position',[obj_SA.z1_mp(ii)-L_SA_1/2,obj_SA.z2_mp(ii)-L_SA_2/2,L_SA_1,L_SA_2],'FaceColor',color,'EdgeColor','c')
    end
    colormap(cmap_par)
    colorbar
    caxis([x_min x_max])
    axis equal tight;
    box on
    xlabel('$z_1$ [m]','Interpreter','latex')
    ylabel('$z_2$ [m]','Interpreter','latex')
    title('5\%-Quantilwert der SA-Zufallsvariablen [MPa]','Interpreter','latex')
    
    % Einstellung der Figure
    figure; hold on;
    set(gcf,'renderer','Painters')
    set(gcf,'color','w');
    
    % Plot des 95%-Quantils
    for ii=1:n_SA_tot
        color=cmap_par(c_id_95p(ii),:);
        rectangle('Position',[obj_SA.z1_mp(ii)-L_SA_1/2,obj_SA.z2_mp(ii)-L_SA_2/2,L_SA_1,L_SA_2],'FaceColor',color,'EdgeColor','c')
    end
    colormap(cmap_par)
    colorbar
    caxis([x_min x_max])
    axis equal tight;
    box on
    xlabel('$z_1$ [m]','Interpreter','latex')
    ylabel('$z_2$ [m]','Interpreter','latex')
    title('95\%-Quantilwert der SA-Zufallsvariablen [MPa]','Interpreter','latex')
    
end

if plot_real>0
    cmap_real=flipud(cool);
    x_real = obj_SA.f_rand(plot_real);
    c_range_real = (max(x_real,[],'all')-min(x_real,[],'all'));
    
    % Plot der Ergebnisse
    for jj = 1:plot_real
        c_id_real   = max(1,round((x_real(:,jj)-min(x_real,[],'all'))/c_range_real*256));
        
        % Einstellung der Figure
        figure; hold on;
        set(gcf,'renderer','Painters')
        set(gcf,'color','w');
        % Plot der Realisierung
        for ii=1:n_SA_tot
            color=cmap_real(c_id_real(ii),:);
            rectangle('Position',[obj_SA.z1_mp(ii)-L_SA_1/2,obj_SA.z2_mp(ii)-L_SA_2/2,L_SA_1,L_SA_2],'FaceColor',color,'EdgeColor','k')
        end
        colormap(cmap_real)
        colorbar
        caxis([min(x_real,[],'all') max(x_real,[],'all')])
        axis equal tight;
        box on
        xlabel('$z_1$ [m]','Interpreter','latex')
        ylabel('$z_2$ [m]','Interpreter','latex')
        title('Stichprobe der SA-Zufallsvariablen [MPa]','Interpreter','latex')
        
    end
    
end

